@@ -248,6 +248,7 @@ class AbodeEntity(entity.Entity):
     """Representation of an Abode entity."""
 
     _attr_attribution = ATTRIBUTION
+    _attr_has_entity_name = True
 
     def __init__(self, data: AbodeSystem) -> None:
         """Initialize Abode entity."""
@@ -283,7 +284,6 @@ class AbodeDevice(AbodeEntity):
         """Initialize Abode device."""
         super().__init__(data)
         self._device = device
-        self._attr_name = device.name
         self._attr_unique_id = device.device_uuid
 
     async def async_added_to_hass(self) -> None:
