@@ -1,13 +1,19 @@
 """Support for August devices."""
+from __future__ import annotations
+
 import asyncio
+from collections.abc import ValuesView
 from itertools import chain
 import logging
 
 from aiohttp import ClientError, ClientResponseError
+from yalexs.doorbell import Doorbell, DoorbellDetail
 from yalexs.exceptions import AugustApiAIOHTTPError
+from yalexs.lock import Lock, LockDetail
 from yalexs.pubnub_activity import activities_from_pubnub_message
 from yalexs.pubnub_async import AugustPubNub, async_create_pubnub
 
+from homeassistant.components import yalexs_ble
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import CONF_PASSWORD
 from homeassistant.core import HomeAssistant, callback
@@ -16,24 +22,25 @@ from homeassistant.exceptions import (
     ConfigEntryNotReady,
     HomeAssistantError,
 )
+from homeassistant.helpers import device_registry as dr
 
 from .activity import ActivityStream
-from .const import DATA_AUGUST, DOMAIN, MIN_TIME_BETWEEN_DETAIL_UPDATES, PLATFORMS
+from .const import DOMAIN, MIN_TIME_BETWEEN_DETAIL_UPDATES, PLATFORMS
 from .exceptions import CannotConnect, InvalidAuth, RequireValidation
 from .gateway import AugustGateway
 from .subscriber import AugustSubscriberMixin
 
 _LOGGER = logging.getLogger(__name__)
 
-API_CACHED_ATTRS = (
+API_CACHED_ATTRS = {
     "door_state",
     "door_state_datetime",
     "lock_status",
     "lock_status_datetime",
-)
+}
 
 
-async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up August from a config entry."""
 
     august_gateway = AugustGateway(hass)
@@ -43,14 +50,17 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry):
         return await async_setup_august(hass, entry, august_gateway)
     except (RequireValidation, InvalidAuth) as err:
         raise ConfigEntryAuthFailed from err
-    except (ClientResponseError, CannotConnect, asyncio.TimeoutError) as err:
+    except asyncio.TimeoutError as err:
+        raise ConfigEntryNotReady("Timed out connecting to august api") from err
+    except (AugustApiAIOHTTPError, ClientResponseError, CannotConnect) as err:
         raise ConfigEntryNotReady from err
 
 
-async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
+async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
 
-    hass.data[DOMAIN][entry.entry_id][DATA_AUGUST].async_stop()
+    data: AugustData = hass.data[DOMAIN][entry.entry_id]
+    data.async_stop()
 
     unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
 
@@ -60,7 +70,9 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry):
     return unload_ok
 
 
-async def async_setup_august(hass, config_entry, august_gateway):
+async def async_setup_august(
+    hass: HomeAssistant, config_entry: ConfigEntry, august_gateway: AugustGateway
+) -> bool:
     """Set up the August component."""
 
     if CONF_PASSWORD in config_entry.data:
@@ -71,18 +83,37 @@ async def async_setup_august(hass, config_entry, august_gateway):
         hass.config_entries.async_update_entry(config_entry, data=config_data)
 
     await august_gateway.async_authenticate()
+    await august_gateway.async_refresh_access_token_if_needed()
 
     hass.data.setdefault(DOMAIN, {})
-    data = hass.data[DOMAIN][config_entry.entry_id] = {
-        DATA_AUGUST: AugustData(hass, august_gateway)
-    }
-    await data[DATA_AUGUST].async_setup()
+    data = hass.data[DOMAIN][config_entry.entry_id] = AugustData(hass, august_gateway)
+    await data.async_setup()
 
-    hass.config_entries.async_setup_platforms(config_entry, PLATFORMS)
+    await hass.config_entries.async_forward_entry_setups(config_entry, PLATFORMS)
 
     return True
 
 
+@callback
+def _async_trigger_ble_lock_discovery(
+    hass: HomeAssistant, locks_with_offline_keys: list[LockDetail]
+):
+    """Update keys for the yalexs-ble integration if available."""
+    for lock_detail in locks_with_offline_keys:
+        yalexs_ble.async_discovery(
+            hass,
+            yalexs_ble.YaleXSBLEDiscovery(
+                {
+                    "name": lock_detail.device_name,
+                    "address": lock_detail.mac_address,
+                    "serial": lock_detail.serial_number,
+                    "key": lock_detail.offline_key,
+                    "slot": lock_detail.offline_slot,
+                }
+            ),
+        )
+
+
 class AugustData(AugustSubscriberMixin):
     """August data object."""
 
@@ -102,11 +133,10 @@ class AugustData(AugustSubscriberMixin):
     async def async_setup(self):
         """Async setup of august device data and activities."""
         token = self._august_gateway.access_token
-        user_data, locks, doorbells = await asyncio.gather(
-            self._api.async_get_user(token),
-            self._api.async_get_operable_locks(token),
-            self._api.async_get_doorbells(token),
-        )
+        # This used to be a gather but it was less reliable with august's recent api changes.
+        user_data = await self._api.async_get_user(token)
+        locks = await self._api.async_get_operable_locks(token)
+        doorbells = await self._api.async_get_doorbells(token)
         if not doorbells:
             doorbells = []
         if not locks:
@@ -124,6 +154,19 @@ class AugustData(AugustSubscriberMixin):
         # detail as we cannot determine if they are usable.
         # This also allows us to avoid checking for
         # detail being None all over the place
+
+        # Currently we know how to feed data to yalexe_ble
+        # but we do not know how to send it to homekit_controller
+        # yet
+        _async_trigger_ble_lock_discovery(
+            self._hass,
+            [
+                lock_detail
+                for lock_detail in self._device_detail_by_id.values()
+                if isinstance(lock_detail, LockDetail) and lock_detail.offline_key
+            ],
+        )
+
         self._remove_inoperative_locks()
         self._remove_inoperative_doorbells()
 
@@ -138,6 +181,36 @@ class AugustData(AugustSubscriberMixin):
         pubnub.subscribe(self.async_pubnub_message)
         self._pubnub_unsub = async_create_pubnub(user_data["UserID"], pubnub)
 
+        if self._locks_by_id:
+            # Do not prevent setup as the sync can timeout
+            # but it is not a fatal error as the lock
+            # will recover automatically when it comes back online.
+            asyncio.create_task(self._async_initial_sync())
+
+    async def _async_initial_sync(self):
+        """Attempt to request an initial sync."""
+        # We don't care if this fails because we only want to wake
+        # locks that are actually online anyways and they will be
+        # awake when they come back online
+        for result in await asyncio.gather(
+            *[
+                self.async_status_async(
+                    device_id, bool(detail.bridge and detail.bridge.hyper_bridge)
+                )
+                for device_id, detail in self._device_detail_by_id.items()
+                if device_id in self._locks_by_id
+            ],
+            return_exceptions=True,
+        ):
+            if isinstance(result, Exception) and not isinstance(
+                result, (asyncio.TimeoutError, ClientResponseError, CannotConnect)
+            ):
+                _LOGGER.warning(
+                    "Unexpected exception during initial sync: %s",
+                    result,
+                    exc_info=result,
+                )
+
     @callback
     def async_pubnub_message(self, device_id, date_time, message):
         """Process a pubnub message."""
@@ -155,16 +228,16 @@ class AugustData(AugustSubscriberMixin):
         self.activity_stream.async_stop()
 
     @property
-    def doorbells(self):
+    def doorbells(self) -> ValuesView[Doorbell]:
         """Return a list of py-august Doorbell objects."""
         return self._doorbells_by_id.values()
 
     @property
-    def locks(self):
+    def locks(self) -> ValuesView[Lock]:
         """Return a list of py-august Lock objects."""
         return self._locks_by_id.values()
 
-    def get_device_detail(self, device_id):
+    def get_device_detail(self, device_id: str) -> DoorbellDetail | LockDetail:
         """Return the py-august LockDetail or DoorbellDetail object for a device."""
         return self._device_detail_by_id[device_id]
 
@@ -172,12 +245,28 @@ class AugustData(AugustSubscriberMixin):
         await self._async_refresh_device_detail_by_ids(self._subscriptions.keys())
 
     async def _async_refresh_device_detail_by_ids(self, device_ids_list):
-        await asyncio.gather(
-            *[
-                self._async_refresh_device_detail_by_id(device_id)
-                for device_id in device_ids_list
-            ]
-        )
+        """Refresh each device in sequence.
+
+        This used to be a gather but it was less reliable with august's
+        recent api changes.
+
+        The august api has been timing out for some devices so
+        we want the ones that it isn't timing out for to keep working.
+        """
+        for device_id in device_ids_list:
+            try:
+                await self._async_refresh_device_detail_by_id(device_id)
+            except asyncio.TimeoutError:
+                _LOGGER.warning(
+                    "Timed out calling august api during refresh of device: %s",
+                    device_id,
+                )
+            except (ClientResponseError, CannotConnect) as err:
+                _LOGGER.warning(
+                    "Error from august api during refresh of device: %s",
+                    device_id,
+                    exc_info=err,
+                )
 
     async def _async_refresh_device_detail_by_id(self, device_id):
         if device_id in self._locks_by_id:
@@ -229,12 +318,15 @@ class AugustData(AugustSubscriberMixin):
             device.device_id,
         )
 
-    def _get_device_name(self, device_id):
+    def get_device(self, device_id: str) -> Doorbell | Lock | None:
+        """Get a device by id."""
+        return self._locks_by_id.get(device_id) or self._doorbells_by_id.get(device_id)
+
+    def _get_device_name(self, device_id: str) -> str | None:
         """Return doorbell or lock name as August has it stored."""
-        if device_id in self._locks_by_id:
-            return self._locks_by_id[device_id].device_name
-        if device_id in self._doorbells_by_id:
-            return self._doorbells_by_id[device_id].device_name
+        if device := self.get_device(device_id):
+            return device.device_name
+        return None
 
     async def async_lock(self, device_id):
         """Lock the device."""
@@ -245,6 +337,26 @@ class AugustData(AugustSubscriberMixin):
             device_id,
         )
 
+    async def async_status_async(self, device_id, hyper_bridge):
+        """Request status of the the device but do not wait for a response since it will come via pubnub."""
+        return await self._async_call_api_op_requires_bridge(
+            device_id,
+            self._api.async_status_async,
+            self._august_gateway.access_token,
+            device_id,
+            hyper_bridge,
+        )
+
+    async def async_lock_async(self, device_id, hyper_bridge):
+        """Lock the device but do not wait for a response since it will come via pubnub."""
+        return await self._async_call_api_op_requires_bridge(
+            device_id,
+            self._api.async_lock_async,
+            self._august_gateway.access_token,
+            device_id,
+            hyper_bridge,
+        )
+
     async def async_unlock(self, device_id):
         """Unlock the device."""
         return await self._async_call_api_op_requires_bridge(
@@ -254,6 +366,16 @@ class AugustData(AugustSubscriberMixin):
             device_id,
         )
 
+    async def async_unlock_async(self, device_id, hyper_bridge):
+        """Unlock the device but do not wait for a response since it will come via pubnub."""
+        return await self._async_call_api_op_requires_bridge(
+            device_id,
+            self._api.async_unlock_async,
+            self._august_gateway.access_token,
+            device_id,
+            hyper_bridge,
+        )
+
     async def _async_call_api_op_requires_bridge(
         self, device_id, func, *args, **kwargs
     ):
@@ -272,19 +394,13 @@ class AugustData(AugustSubscriberMixin):
     def _remove_inoperative_doorbells(self):
         for doorbell in list(self.doorbells):
             device_id = doorbell.device_id
-            doorbell_is_operative = False
-            doorbell_detail = self._device_detail_by_id.get(device_id)
-            if doorbell_detail is None:
-                _LOGGER.info(
-                    "The doorbell %s could not be setup because the system could not fetch details about the doorbell",
-                    doorbell.device_name,
-                )
-            else:
-                doorbell_is_operative = True
-
-            if not doorbell_is_operative:
-                del self._doorbells_by_id[device_id]
-                del self._device_detail_by_id[device_id]
+            if self._device_detail_by_id.get(device_id):
+                continue
+            _LOGGER.info(
+                "The doorbell %s could not be setup because the system could not fetch details about the doorbell",
+                doorbell.device_name,
+            )
+            del self._doorbells_by_id[device_id]
 
     def _remove_inoperative_locks(self):
         # Remove non-operative locks as there must
@@ -292,7 +408,6 @@ class AugustData(AugustSubscriberMixin):
         # be usable
         for lock in list(self.locks):
             device_id = lock.device_id
-            lock_is_operative = False
             lock_detail = self._device_detail_by_id.get(device_id)
             if lock_detail is None:
                 _LOGGER.info(
@@ -304,14 +419,12 @@ class AugustData(AugustSubscriberMixin):
                     "The lock %s could not be setup because it does not have a bridge (Connect)",
                     lock.device_name,
                 )
+                del self._device_detail_by_id[device_id]
             # Bridge may come back online later so we still add the device since we will
             # have a pubnub subscription to tell use when it recovers
             else:
-                lock_is_operative = True
-
-            if not lock_is_operative:
-                del self._locks_by_id[device_id]
-                del self._device_detail_by_id[device_id]
+                continue
+            del self._locks_by_id[device_id]
 
 
 def _save_live_attrs(lock_detail):
@@ -328,3 +441,15 @@ def _restore_live_attrs(lock_detail, attrs):
     """Restore the non-cache attributes after a cached update."""
     for attr, value in attrs.items():
         setattr(lock_detail, attr, value)
+
+
+async def async_remove_config_entry_device(
+    hass: HomeAssistant, config_entry: ConfigEntry, device_entry: dr.DeviceEntry
+) -> bool:
+    """Remove august config entry from a device if its no longer present."""
+    data: AugustData = hass.data[DOMAIN][config_entry.entry_id]
+    return not any(
+        identifier
+        for identifier in device_entry.identifiers
+        if identifier[0] == DOMAIN and data.get_device(identifier[1])
+    )
