@@ -1,13 +1,16 @@
 """Support for Avion dimmers."""
+from __future__ import annotations
+
 import importlib
 import time
+from typing import Any
 
 import voluptuous as vol
 
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
     PLATFORM_SCHEMA,
-    SUPPORT_BRIGHTNESS,
+    ColorMode,
     LightEntity,
 )
 from homeassistant.const import (
@@ -18,9 +21,10 @@ from homeassistant.const import (
     CONF_PASSWORD,
     CONF_USERNAME,
 )
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
-
-SUPPORT_AVION_LED = SUPPORT_BRIGHTNESS
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
 DEVICE_SCHEMA = vol.Schema(
     {
@@ -39,7 +43,12 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up an Avion switch."""
     avion = importlib.import_module("avion")
 
@@ -65,9 +74,11 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
 class AvionLight(LightEntity):
     """Representation of an Avion light."""
 
-    _attr_supported_features = SUPPORT_AVION_LED
+    _attr_support_color_mode = ColorMode.BRIGHTNESS
+    _attr_support_color_modes = {ColorMode.BRIGHTNESS}
     _attr_should_poll = False
     _attr_assumed_state = True
+    _attr_is_on = True
 
     def __init__(self, device):
         """Initialize the light."""
@@ -93,7 +104,7 @@ class AvionLight(LightEntity):
                 self._switch.connect()
         return True
 
-    def turn_on(self, **kwargs):
+    def turn_on(self, **kwargs: Any) -> None:
         """Turn the specified or all lights on."""
         if (brightness := kwargs.get(ATTR_BRIGHTNESS)) is not None:
             self._attr_brightness = brightness
@@ -101,7 +112,7 @@ class AvionLight(LightEntity):
         self.set_state(self.brightness)
         self._attr_is_on = True
 
-    def turn_off(self, **kwargs):
+    def turn_off(self, **kwargs: Any) -> None:
         """Turn the specified or all lights off."""
         self.set_state(0)
         self._attr_is_on = False
