@@ -1,9 +1,12 @@
 """Blueprint models."""
+from __future__ import annotations
+
 import asyncio
+from collections.abc import Callable
 import logging
 import pathlib
 import shutil
-from typing import Any, Dict, List, Optional, Union
+from typing import Any
 
 from awesomeversion import AwesomeVersion
 import voluptuous as vol
@@ -33,6 +36,7 @@ from .const import (
 )
 from .errors import (
     BlueprintException,
+    BlueprintInUse,
     FailedToLoad,
     FileAlreadyExists,
     InvalidBlueprint,
@@ -49,8 +53,8 @@ class Blueprint:
         self,
         data: dict,
         *,
-        path: Optional[str] = None,
-        expected_domain: Optional[str] = None,
+        path: str | None = None,
+        expected_domain: str | None = None,
     ) -> None:
         """Initialize a blueprint."""
         try:
@@ -95,7 +99,7 @@ class Blueprint:
         """Return blueprint metadata."""
         return self.data[CONF_BLUEPRINT]
 
-    def update_metadata(self, *, source_url: Optional[str] = None) -> None:
+    def update_metadata(self, *, source_url: str | None = None) -> None:
         """Update metadata."""
         if source_url is not None:
             self.data[CONF_BLUEPRINT][CONF_SOURCE_URL] = source_url
@@ -105,7 +109,7 @@ class Blueprint:
         return yaml.dump(self.data)
 
     @callback
-    def validate(self) -> Optional[List[str]]:
+    def validate(self) -> list[str] | None:
         """Test if the Home Assistant installation supports this blueprint.
 
         Return list of errors if not valid.
@@ -126,7 +130,7 @@ class BlueprintInputs:
     """Inputs for a blueprint."""
 
     def __init__(
-        self, blueprint: Blueprint, config_with_inputs: Dict[str, Any]
+        self, blueprint: Blueprint, config_with_inputs: dict[str, Any]
     ) -> None:
         """Instantiate a blueprint inputs object."""
         self.blueprint = blueprint
@@ -181,12 +185,14 @@ class DomainBlueprints:
         hass: HomeAssistant,
         domain: str,
         logger: logging.Logger,
+        blueprint_in_use: Callable[[HomeAssistant, str], bool],
     ) -> None:
         """Initialize a domain blueprints instance."""
         self.hass = hass
         self.domain = domain
         self.logger = logger
-        self._blueprints = {}
+        self._blueprint_in_use = blueprint_in_use
+        self._blueprints: dict[str, Blueprint | None] = {}
         self._load_lock = asyncio.Lock()
 
         hass.data.setdefault(DOMAIN, {})[domain] = self
@@ -214,19 +220,20 @@ class DomainBlueprints:
         except HomeAssistantError as err:
             raise FailedToLoad(self.domain, blueprint_path, err) from err
 
+        assert isinstance(blueprint_data, dict)
         return Blueprint(
             blueprint_data, expected_domain=self.domain, path=blueprint_path
         )
 
-    def _load_blueprints(self) -> Dict[str, Union[Blueprint, BlueprintException]]:
+    def _load_blueprints(self) -> dict[str, Blueprint | BlueprintException | None]:
         """Load all the blueprints."""
         blueprint_folder = pathlib.Path(
             self.hass.config.path(BLUEPRINT_FOLDER, self.domain)
         )
-        results = {}
+        results: dict[str, Blueprint | BlueprintException | None] = {}
 
-        for blueprint_path in blueprint_folder.glob("**/*.yaml"):
-            blueprint_path = str(blueprint_path.relative_to(blueprint_folder))
+        for path in blueprint_folder.glob("**/*.yaml"):
+            blueprint_path = str(path.relative_to(blueprint_folder))
             if self._blueprints.get(blueprint_path) is None:
                 try:
                     self._blueprints[blueprint_path] = self._load_blueprint(
@@ -243,7 +250,7 @@ class DomainBlueprints:
 
     async def async_get_blueprints(
         self,
-    ) -> Dict[str, Union[Blueprint, BlueprintException]]:
+    ) -> dict[str, Blueprint | BlueprintException | None]:
         """Get all the blueprints."""
         async with self._load_lock:
             return await self.hass.async_add_executor_job(self._load_blueprints)
@@ -253,8 +260,7 @@ class DomainBlueprints:
 
         def load_from_cache():
             """Load blueprint from cache."""
-            blueprint = self._blueprints[blueprint_path]
-            if blueprint is None:
+            if (blueprint := self._blueprints[blueprint_path]) is None:
                 raise FailedToLoad(
                     self.domain,
                     blueprint_path,
@@ -300,6 +306,8 @@ class DomainBlueprints:
 
     async def async_remove_blueprint(self, blueprint_path: str) -> None:
         """Remove a blueprint file."""
+        if self._blueprint_in_use(self.hass, blueprint_path):
+            raise BlueprintInUse(self.domain, blueprint_path)
         path = self.blueprint_folder / blueprint_path
         await self.hass.async_add_executor_job(path.unlink)
         self._blueprints[blueprint_path] = None
@@ -314,7 +322,7 @@ class DomainBlueprints:
             raise FileAlreadyExists(self.domain, blueprint_path)
 
         path.parent.mkdir(parents=True, exist_ok=True)
-        path.write_text(blueprint.yaml())
+        path.write_text(blueprint.yaml(), encoding="utf-8")
 
     async def async_add_blueprint(
         self, blueprint: Blueprint, blueprint_path: str
