@@ -5,7 +5,8 @@ from __future__ import annotations
 import voluptuous as vol
 
 from homeassistant import data_entry_flow
-from homeassistant.components.repairs import RepairsFlow
+from homeassistant.components.repairs import ConfirmRepairFlow, RepairsFlow
+from homeassistant.core import HomeAssistant
 
 
 class DemoFixFlow(RepairsFlow):
@@ -23,11 +24,20 @@ class DemoFixFlow(RepairsFlow):
     ) -> data_entry_flow.FlowResult:
         """Handle the confirm step of a fix flow."""
         if user_input is not None:
-            return self.async_create_entry(title="Fixed issue", data={})
+            return self.async_create_entry(title="", data={})
 
         return self.async_show_form(step_id="confirm", data_schema=vol.Schema({}))
 
 
-async def async_create_fix_flow(hass, issue_id):
+async def async_create_fix_flow(
+    hass: HomeAssistant,
+    issue_id: str,
+    data: dict[str, str | int | float | None] | None,
+) -> RepairsFlow:
     """Create flow."""
+    if issue_id == "bad_psu":
+        # The bad_psu issue doesn't have its own flow
+        return ConfirmRepairFlow()
+
+    # Other issues have a custom flow
     return DemoFixFlow()
