@@ -8,7 +8,7 @@ from datetime import datetime, timedelta
 import functools
 from typing import Any, TypeVar
 
-from async_upnp_client import UpnpService, UpnpStateVariable
+from async_upnp_client.client import UpnpService, UpnpStateVariable
 from async_upnp_client.const import NotificationSubType
 from async_upnp_client.exceptions import UpnpError, UpnpResponseError
 from async_upnp_client.profiles.dlna import DmrDevice, PlayMode, TransportState
@@ -19,44 +19,22 @@ from typing_extensions import Concatenate, ParamSpec
 from homeassistant import config_entries
 from homeassistant.components import media_source, ssdp
 from homeassistant.components.media_player import (
+    ATTR_MEDIA_EXTRA,
     BrowseMedia,
     MediaPlayerEntity,
+    MediaPlayerEntityFeature,
+    MediaPlayerState,
+    MediaType,
+    RepeatMode,
     async_process_play_media_url,
 )
-from homeassistant.components.media_player.const import (
-    ATTR_MEDIA_EXTRA,
-    REPEAT_MODE_ALL,
-    REPEAT_MODE_OFF,
-    REPEAT_MODE_ONE,
-    SUPPORT_BROWSE_MEDIA,
-    SUPPORT_NEXT_TRACK,
-    SUPPORT_PAUSE,
-    SUPPORT_PLAY,
-    SUPPORT_PLAY_MEDIA,
-    SUPPORT_PREVIOUS_TRACK,
-    SUPPORT_REPEAT_SET,
-    SUPPORT_SEEK,
-    SUPPORT_SELECT_SOUND_MODE,
-    SUPPORT_SHUFFLE_SET,
-    SUPPORT_STOP,
-    SUPPORT_VOLUME_MUTE,
-    SUPPORT_VOLUME_SET,
-)
-from homeassistant.const import (
-    CONF_DEVICE_ID,
-    CONF_TYPE,
-    CONF_URL,
-    STATE_IDLE,
-    STATE_OFF,
-    STATE_ON,
-    STATE_PAUSED,
-    STATE_PLAYING,
-)
+from homeassistant.const import CONF_DEVICE_ID, CONF_TYPE, CONF_URL
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers import device_registry, entity_registry
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import (
+    CONF_BROWSE_UNFILTERED,
     CONF_CALLBACK_URL_OVERRIDE,
     CONF_LISTEN_PORT,
     CONF_POLL_AVAILABILITY,
@@ -73,20 +51,20 @@ from .data import EventListenAddr, get_domain_data
 
 PARALLEL_UPDATES = 0
 
-_T = TypeVar("_T", bound="DlnaDmrEntity")
+_DlnaDmrEntityT = TypeVar("_DlnaDmrEntityT", bound="DlnaDmrEntity")
 _R = TypeVar("_R")
 _P = ParamSpec("_P")
 
-Func = TypeVar("Func", bound=Callable[..., Any])
-
 
 def catch_request_errors(
-    func: Callable[Concatenate[_T, _P], Awaitable[_R]]  # type: ignore[misc]
-) -> Callable[Concatenate[_T, _P], Coroutine[Any, Any, _R | None]]:  # type: ignore[misc]
+    func: Callable[Concatenate[_DlnaDmrEntityT, _P], Awaitable[_R]]
+) -> Callable[Concatenate[_DlnaDmrEntityT, _P], Coroutine[Any, Any, _R | None]]:
     """Catch UpnpError errors."""
 
     @functools.wraps(func)
-    async def wrapper(self: _T, *args: _P.args, **kwargs: _P.kwargs) -> _R | None:
+    async def wrapper(
+        self: _DlnaDmrEntityT, *args: _P.args, **kwargs: _P.kwargs
+    ) -> _R | None:
         """Catch UpnpError errors and check availability before and after request."""
         if not self.available:
             _LOGGER.warning(
@@ -94,7 +72,7 @@ def catch_request_errors(
             )
             return None
         try:
-            return await func(self, *args, **kwargs)  # type: ignore[no-any-return]  # mypy can't yet infer 'func'
+            return await func(self, *args, **kwargs)
         except UpnpError as err:
             self.check_available = True
             _LOGGER.error("Error during call %s: %r", func.__name__, err)
@@ -120,6 +98,7 @@ async def async_setup_entry(
         event_callback_url=entry.options.get(CONF_CALLBACK_URL_OVERRIDE),
         poll_availability=entry.options.get(CONF_POLL_AVAILABILITY, False),
         location=entry.data[CONF_URL],
+        browse_unfiltered=entry.options.get(CONF_BROWSE_UNFILTERED, False),
     )
 
     async_add_entities([entity])
@@ -136,6 +115,8 @@ class DlnaDmrEntity(MediaPlayerEntity):
     # Last known URL for the device, used when adding this entity to hass to try
     # to connect before SSDP has rediscovered it, or when SSDP discovery fails.
     location: str
+    # Should the async_browse_media function *not* filter out incompatible media?
+    browse_unfiltered: bool
 
     _device_lock: asyncio.Lock  # Held when connecting or disconnecting the device
     _device: DmrDevice | None = None
@@ -158,6 +139,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         event_callback_url: str | None,
         poll_availability: bool,
         location: str,
+        browse_unfiltered: bool,
     ) -> None:
         """Initialize DLNA DMR entity."""
         self.udn = udn
@@ -166,6 +148,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         self._event_addr = EventListenAddr(None, event_port, event_callback_url)
         self.poll_availability = poll_availability
         self.location = location
+        self.browse_unfiltered = browse_unfiltered
         self._device_lock = asyncio.Lock()
 
     async def async_added_to_hass(self) -> None:
@@ -287,6 +270,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         )
         self.location = entry.data[CONF_URL]
         self.poll_availability = entry.options.get(CONF_POLL_AVAILABILITY, False)
+        self.browse_unfiltered = entry.options.get(CONF_BROWSE_UNFILTERED, False)
 
         new_port = entry.options.get(CONF_LISTEN_PORT) or 0
         new_callback_url = entry.options.get(CONF_CALLBACK_URL_OVERRIDE)
@@ -471,27 +455,27 @@ class DlnaDmrEntity(MediaPlayerEntity):
         return f"{self.udn}::{self.device_type}"
 
     @property
-    def state(self) -> str | None:
+    def state(self) -> MediaPlayerState | None:
         """State of the player."""
         if not self._device or not self.available:
-            return STATE_OFF
+            return MediaPlayerState.OFF
         if self._device.transport_state is None:
-            return STATE_ON
+            return MediaPlayerState.ON
         if self._device.transport_state in (
             TransportState.PLAYING,
             TransportState.TRANSITIONING,
         ):
-            return STATE_PLAYING
+            return MediaPlayerState.PLAYING
         if self._device.transport_state in (
             TransportState.PAUSED_PLAYBACK,
             TransportState.PAUSED_RECORDING,
         ):
-            return STATE_PAUSED
+            return MediaPlayerState.PAUSED
         if self._device.transport_state == TransportState.VENDOR_DEFINED:
             # Unable to map this state to anything reasonable, so it's "Unknown"
             return None
 
-        return STATE_IDLE
+        return MediaPlayerState.IDLE
 
     @property
     def supported_features(self) -> int:
@@ -505,32 +489,35 @@ class DlnaDmrEntity(MediaPlayerEntity):
         supported_features = 0
 
         if self._device.has_volume_level:
-            supported_features |= SUPPORT_VOLUME_SET
+            supported_features |= MediaPlayerEntityFeature.VOLUME_SET
         if self._device.has_volume_mute:
-            supported_features |= SUPPORT_VOLUME_MUTE
+            supported_features |= MediaPlayerEntityFeature.VOLUME_MUTE
         if self._device.can_play:
-            supported_features |= SUPPORT_PLAY
+            supported_features |= MediaPlayerEntityFeature.PLAY
         if self._device.can_pause:
-            supported_features |= SUPPORT_PAUSE
+            supported_features |= MediaPlayerEntityFeature.PAUSE
         if self._device.can_stop:
-            supported_features |= SUPPORT_STOP
+            supported_features |= MediaPlayerEntityFeature.STOP
         if self._device.can_previous:
-            supported_features |= SUPPORT_PREVIOUS_TRACK
+            supported_features |= MediaPlayerEntityFeature.PREVIOUS_TRACK
         if self._device.can_next:
-            supported_features |= SUPPORT_NEXT_TRACK
+            supported_features |= MediaPlayerEntityFeature.NEXT_TRACK
         if self._device.has_play_media:
-            supported_features |= SUPPORT_PLAY_MEDIA | SUPPORT_BROWSE_MEDIA
+            supported_features |= (
+                MediaPlayerEntityFeature.PLAY_MEDIA
+                | MediaPlayerEntityFeature.BROWSE_MEDIA
+            )
         if self._device.can_seek_rel_time:
-            supported_features |= SUPPORT_SEEK
+            supported_features |= MediaPlayerEntityFeature.SEEK
 
         play_modes = self._device.valid_play_modes
         if play_modes & {PlayMode.RANDOM, PlayMode.SHUFFLE}:
-            supported_features |= SUPPORT_SHUFFLE_SET
+            supported_features |= MediaPlayerEntityFeature.SHUFFLE_SET
         if play_modes & {PlayMode.REPEAT_ONE, PlayMode.REPEAT_ALL}:
-            supported_features |= SUPPORT_REPEAT_SET
+            supported_features |= MediaPlayerEntityFeature.REPEAT_SET
 
         if self._device.has_presets:
-            supported_features |= SUPPORT_SELECT_SOUND_MODE
+            supported_features |= MediaPlayerEntityFeature.SELECT_SOUND_MODE
 
         return supported_features
 
@@ -580,7 +567,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         await self._device.async_stop()
 
     @catch_request_errors
-    async def async_media_seek(self, position: int | float) -> None:
+    async def async_media_seek(self, position: float) -> None:
         """Send seek command."""
         assert self._device is not None
         time = timedelta(seconds=position)
@@ -588,7 +575,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
 
     @catch_request_errors
     async def async_play_media(
-        self, media_type: str, media_id: str, **kwargs: Any
+        self, media_type: MediaType | str, media_id: str, **kwargs: Any
     ) -> None:
         """Play a piece of media."""
         _LOGGER.debug("Playing media: %s, %s, %s", media_type, media_id, kwargs)
@@ -599,7 +586,9 @@ class DlnaDmrEntity(MediaPlayerEntity):
 
         # If media is media_source, resolve it to url and MIME type, and maybe metadata
         if media_source.is_media_source_id(media_id):
-            sourced_media = await media_source.async_resolve_media(self.hass, media_id)
+            sourced_media = await media_source.async_resolve_media(
+                self.hass, media_id, self.entity_id
+            )
             media_type = sourced_media.mime_type
             media_id = sourced_media.url
             _LOGGER.debug("sourced_media is %s", sourced_media)
@@ -683,7 +672,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         """Enable/disable shuffle mode."""
         assert self._device is not None
 
-        repeat = self.repeat or REPEAT_MODE_OFF
+        repeat = self.repeat or RepeatMode.OFF
         potential_play_modes = SHUFFLE_PLAY_MODES[(shuffle, repeat)]
 
         valid_play_modes = self._device.valid_play_modes
@@ -698,7 +687,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         )
 
     @property
-    def repeat(self) -> str | None:
+    def repeat(self) -> RepeatMode | None:
         """Return current repeat mode."""
         if not self._device:
             return None
@@ -710,15 +699,15 @@ class DlnaDmrEntity(MediaPlayerEntity):
             return None
 
         if play_mode == PlayMode.REPEAT_ONE:
-            return REPEAT_MODE_ONE
+            return RepeatMode.ONE
 
         if play_mode in (PlayMode.REPEAT_ALL, PlayMode.RANDOM):
-            return REPEAT_MODE_ALL
+            return RepeatMode.ALL
 
-        return REPEAT_MODE_OFF
+        return RepeatMode.OFF
 
     @catch_request_errors
-    async def async_set_repeat(self, repeat: str) -> None:
+    async def async_set_repeat(self, repeat: RepeatMode) -> None:
         """Set repeat mode."""
         assert self._device is not None
 
@@ -771,14 +760,21 @@ class DlnaDmrEntity(MediaPlayerEntity):
         # media_content_type is ignored; it's the content_type of the current
         # media_content_id, not the desired content_type of whomever is calling.
 
-        content_filter = self._get_content_filter()
+        if self.browse_unfiltered:
+            content_filter = None
+        else:
+            content_filter = self._get_content_filter()
 
         return await media_source.async_browse_media(
             self.hass, media_content_id, content_filter=content_filter
         )
 
     def _get_content_filter(self) -> Callable[[BrowseMedia], bool]:
-        """Return a function that filters media based on what the renderer can play."""
+        """Return a function that filters media based on what the renderer can play.
+
+        The filtering is pretty loose; it's better to show something that can't
+        be played than hide something that can.
+        """
         if not self._device or not self._device.sink_protocol_info:
             # Nothing is specified by the renderer, so show everything
             _LOGGER.debug("Get content filter with no device or sink protocol info")
@@ -789,18 +785,25 @@ class DlnaDmrEntity(MediaPlayerEntity):
             # Renderer claims it can handle everything, so show everything
             return lambda _: True
 
-        # Convert list of things like "http-get:*:audio/mpeg:*" to just "audio/mpeg"
-        content_types: list[str] = []
+        # Convert list of things like "http-get:*:audio/mpeg;codecs=mp3:*"
+        # to just "audio/mpeg"
+        content_types = set[str]()
         for protocol_info in self._device.sink_protocol_info:
             protocol, _, content_format, _ = protocol_info.split(":", 3)
+            # Transform content_format for better generic matching
+            content_format = content_format.lower().replace("/x-", "/", 1)
+            content_format = content_format.partition(";")[0]
+
             if protocol in STREAMABLE_PROTOCOLS:
-                content_types.append(content_format)
+                content_types.add(content_format)
 
-        def _content_type_filter(item: BrowseMedia) -> bool:
-            """Filter media items by their content_type."""
-            return item.media_content_type in content_types
+        def _content_filter(item: BrowseMedia) -> bool:
+            """Filter media items by their media_content_type."""
+            content_type = item.media_content_type
+            content_type = content_type.lower().replace("/x-", "/", 1).partition(";")[0]
+            return content_type in content_types
 
-        return _content_type_filter
+        return _content_filter
 
     @property
     def media_title(self) -> str | None:
@@ -825,7 +828,7 @@ class DlnaDmrEntity(MediaPlayerEntity):
         return self._device.current_track_uri
 
     @property
-    def media_content_type(self) -> str | None:
+    def media_content_type(self) -> MediaType | None:
         """Content type of current playing media."""
         if not self._device or not self._device.media_class:
             return None
