@@ -1,6 +1,10 @@
 """Support for EnOcean sensors."""
 from __future__ import annotations
 
+from collections.abc import Callable
+from dataclasses import dataclass
+
+from enocean.utils import combine_hex
 import voluptuous as vol
 
 from homeassistant.components.sensor import (
@@ -40,37 +44,56 @@ SENSOR_TYPE_POWER = "powersensor"
 SENSOR_TYPE_TEMPERATURE = "temperature"
 SENSOR_TYPE_WINDOWHANDLE = "windowhandle"
 
-SENSOR_DESC_TEMPERATURE = SensorEntityDescription(
+
+@dataclass
+class EnOceanSensorEntityDescriptionMixin:
+    """Mixin for required keys."""
+
+    unique_id: Callable[[list[int]], str | None]
+
+
+@dataclass
+class EnOceanSensorEntityDescription(
+    SensorEntityDescription, EnOceanSensorEntityDescriptionMixin
+):
+    """Describes EnOcean sensor entity."""
+
+
+SENSOR_DESC_TEMPERATURE = EnOceanSensorEntityDescription(
     key=SENSOR_TYPE_TEMPERATURE,
     name="Temperature",
     native_unit_of_measurement=TEMP_CELSIUS,
     icon="mdi:thermometer",
     device_class=SensorDeviceClass.TEMPERATURE,
     state_class=SensorStateClass.MEASUREMENT,
+    unique_id=lambda dev_id: f"{combine_hex(dev_id)}-{SENSOR_TYPE_TEMPERATURE}",
 )
 
-SENSOR_DESC_HUMIDITY = SensorEntityDescription(
+SENSOR_DESC_HUMIDITY = EnOceanSensorEntityDescription(
     key=SENSOR_TYPE_HUMIDITY,
     name="Humidity",
     native_unit_of_measurement=PERCENTAGE,
     icon="mdi:water-percent",
     device_class=SensorDeviceClass.HUMIDITY,
     state_class=SensorStateClass.MEASUREMENT,
+    unique_id=lambda dev_id: f"{combine_hex(dev_id)}-{SENSOR_TYPE_HUMIDITY}",
 )
 
-SENSOR_DESC_POWER = SensorEntityDescription(
+SENSOR_DESC_POWER = EnOceanSensorEntityDescription(
     key=SENSOR_TYPE_POWER,
     name="Power",
     native_unit_of_measurement=POWER_WATT,
     icon="mdi:power-plug",
     device_class=SensorDeviceClass.POWER,
     state_class=SensorStateClass.MEASUREMENT,
+    unique_id=lambda dev_id: f"{combine_hex(dev_id)}-{SENSOR_TYPE_POWER}",
 )
 
-SENSOR_DESC_WINDOWHANDLE = SensorEntityDescription(
+SENSOR_DESC_WINDOWHANDLE = EnOceanSensorEntityDescription(
     key=SENSOR_TYPE_WINDOWHANDLE,
     name="WindowHandle",
-    icon="mdi:window",
+    icon="mdi:window-open-variant",
+    unique_id=lambda dev_id: f"{combine_hex(dev_id)}-{SENSOR_TYPE_WINDOWHANDLE}",
 )
 
 
@@ -132,13 +155,14 @@ def setup_platform(
 class EnOceanSensor(EnOceanEntity, RestoreEntity, SensorEntity):
     """Representation of an  EnOcean sensor device such as a power meter."""
 
-    def __init__(self, dev_id, dev_name, description: SensorEntityDescription):
+    def __init__(self, dev_id, dev_name, description: EnOceanSensorEntityDescription):
         """Initialize the EnOcean sensor device."""
         super().__init__(dev_id, dev_name)
         self.entity_description = description
         self._attr_name = f"{description.name} {dev_name}"
+        self._attr_unique_id = description.unique_id(dev_id)
 
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Call when entity about to be added to hass."""
         # If not None, we got an initial value.
         await super().async_added_to_hass()
@@ -194,7 +218,7 @@ class EnOceanTemperatureSensor(EnOceanSensor):
         self,
         dev_id,
         dev_name,
-        description: SensorEntityDescription,
+        description: EnOceanSensorEntityDescription,
         *,
         scale_min,
         scale_max,
@@ -248,7 +272,6 @@ class EnOceanWindowHandle(EnOceanSensor):
 
     def value_changed(self, packet):
         """Update the internal state of the sensor."""
-
         action = (packet.data[1] & 0x70) >> 4
 
         if action == 0x07:
