@@ -5,6 +5,7 @@ Such systems include evohome, Round Thermostat, and others.
 from __future__ import annotations
 
 from datetime import datetime as dt, timedelta
+from http import HTTPStatus
 import logging
 import re
 from typing import Any
@@ -19,11 +20,10 @@ from homeassistant.const import (
     CONF_PASSWORD,
     CONF_SCAN_INTERVAL,
     CONF_USERNAME,
-    HTTP_SERVICE_UNAVAILABLE,
-    HTTP_TOO_MANY_REQUESTS,
-    TEMP_CELSIUS,
+    Platform,
 )
-from homeassistant.core import HomeAssistant, callback
+from homeassistant.core import HomeAssistant, ServiceCall, callback
+from homeassistant.helpers import entity_registry as er
 from homeassistant.helpers.aiohttp_client import async_get_clientsession
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.discovery import async_load_platform
@@ -32,7 +32,9 @@ from homeassistant.helpers.dispatcher import (
     async_dispatcher_send,
 )
 from homeassistant.helpers.entity import Entity
+from homeassistant.helpers.event import async_call_later, async_track_time_interval
 from homeassistant.helpers.service import verify_domain_control
+from homeassistant.helpers.storage import Store
 from homeassistant.helpers.typing import ConfigType
 import homeassistant.util.dt as dt_util
 
@@ -111,8 +113,9 @@ def _dt_aware_to_naive(dt_aware: dt) -> dt:
 
 def convert_until(status_dict: dict, until_key: str) -> None:
     """Reformat a dt str from "%Y-%m-%dT%H:%M:%SZ" as local/aware/isoformat."""
-    if until_key in status_dict:  # only present for certain modes
-        dt_utc_naive = dt_util.parse_datetime(status_dict[until_key])
+    if until_key in status_dict and (  # only present for certain modes
+        dt_utc_naive := dt_util.parse_datetime(status_dict[until_key])
+    ):
         status_dict[until_key] = dt_util.as_local(dt_utc_naive).isoformat()
 
 
@@ -123,7 +126,9 @@ def convert_dict(dictionary: dict[str, Any]) -> dict[str, Any]:
         """Convert a string to snake_case."""
         string = re.sub(r"[\-\.\s]", "_", str(key))
         return (string[0]).lower() + re.sub(
-            r"[A-Z]", lambda matched: f"_{matched.group(0).lower()}", string[1:]
+            r"[A-Z]",
+            lambda matched: f"_{matched.group(0).lower()}",  # type:ignore[str-bytes-safe]
+            string[1:],
         )
 
     return {
@@ -134,7 +139,7 @@ def convert_dict(dictionary: dict[str, Any]) -> dict[str, Any]:
     }
 
 
-def _handle_exception(err) -> bool:
+def _handle_exception(err) -> None:
     """Return False if the exception can't be ignored."""
     try:
         raise err
@@ -158,13 +163,13 @@ def _handle_exception(err) -> bool:
         )
 
     except aiohttp.ClientResponseError:
-        if err.status == HTTP_SERVICE_UNAVAILABLE:
+        if err.status == HTTPStatus.SERVICE_UNAVAILABLE:
             _LOGGER.warning(
                 "The vendor says their server is currently unavailable. "
                 "Check the vendor's service status page"
             )
 
-        elif err.status == HTTP_TOO_MANY_REQUESTS:
+        elif err.status == HTTPStatus.TOO_MANY_REQUESTS:
             _LOGGER.warning(
                 "The vendor's API rate limit has been exceeded. "
                 "If this message persists, consider increasing the %s",
@@ -188,15 +193,15 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
             return ({}, None)
 
         # evohomeasync2 requires naive/local datetimes as strings
-        if tokens.get(ACCESS_TOKEN_EXPIRES) is not None:
-            tokens[ACCESS_TOKEN_EXPIRES] = _dt_aware_to_naive(
-                dt_util.parse_datetime(tokens[ACCESS_TOKEN_EXPIRES])
-            )
+        if tokens.get(ACCESS_TOKEN_EXPIRES) is not None and (
+            expires := dt_util.parse_datetime(tokens[ACCESS_TOKEN_EXPIRES])
+        ):
+            tokens[ACCESS_TOKEN_EXPIRES] = _dt_aware_to_naive(expires)
 
         user_data = tokens.pop(USER_DATA, None)
         return (tokens, user_data)
 
-    store = hass.helpers.storage.Store(STORAGE_VER, STORAGE_KEY)
+    store = Store[dict[str, Any]](hass, STORAGE_VER, STORAGE_KEY)
     tokens, user_data = await load_auth_tokens(store)
 
     client_v2 = evohomeasync2.EvohomeClient(
@@ -228,7 +233,10 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
         return False
 
     if _LOGGER.isEnabledFor(logging.DEBUG):
-        _config = {"locationInfo": {"timeZone": None}, GWS: [{TCS: None}]}
+        _config: dict[str, Any] = {
+            "locationInfo": {"timeZone": None},
+            GWS: [{TCS: None}],
+        }
         _config["locationInfo"]["timeZone"] = loc_config["locationInfo"]["timeZone"]
         _config[GWS][0][TCS] = loc_config[GWS][0][TCS]
         _LOGGER.debug("Config = %s", _config)
@@ -248,14 +256,16 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     await broker.save_auth_tokens()
     await broker.async_update()  # get initial state
 
-    hass.async_create_task(async_load_platform(hass, "climate", DOMAIN, {}, config))
+    hass.async_create_task(
+        async_load_platform(hass, Platform.CLIMATE, DOMAIN, {}, config)
+    )
     if broker.tcs.hotwater:
         hass.async_create_task(
-            async_load_platform(hass, "water_heater", DOMAIN, {}, config)
+            async_load_platform(hass, Platform.WATER_HEATER, DOMAIN, {}, config)
         )
 
-    hass.helpers.event.async_track_time_interval(
-        broker.async_update, config[DOMAIN][CONF_SCAN_INTERVAL]
+    async_track_time_interval(
+        hass, broker.async_update, config[DOMAIN][CONF_SCAN_INTERVAL]
     )
 
     setup_service_functions(hass, broker)
@@ -275,12 +285,12 @@ def setup_service_functions(hass: HomeAssistant, broker):
     """
 
     @verify_domain_control(hass, DOMAIN)
-    async def force_refresh(call) -> None:
+    async def force_refresh(call: ServiceCall) -> None:
         """Obtain the latest state data via the vendor's RESTful API."""
         await broker.async_update()
 
     @verify_domain_control(hass, DOMAIN)
-    async def set_system_mode(call) -> None:
+    async def set_system_mode(call: ServiceCall) -> None:
         """Set the system mode."""
         payload = {
             "unique_id": broker.tcs.systemId,
@@ -290,11 +300,11 @@ def setup_service_functions(hass: HomeAssistant, broker):
         async_dispatcher_send(hass, DOMAIN, payload)
 
     @verify_domain_control(hass, DOMAIN)
-    async def set_zone_override(call) -> None:
+    async def set_zone_override(call: ServiceCall) -> None:
         """Set the zone override (setpoint)."""
         entity_id = call.data[ATTR_ENTITY_ID]
 
-        registry = await hass.helpers.entity_registry.async_get_registry()
+        registry = er.async_get(hass)
         registry_entry = registry.async_get(entity_id)
 
         if registry_entry is None or registry_entry.platform != DOMAIN:
@@ -385,7 +395,14 @@ def setup_service_functions(hass: HomeAssistant, broker):
 class EvoBroker:
     """Container for evohome client and data."""
 
-    def __init__(self, hass, client, client_v1, store, params) -> None:
+    def __init__(
+        self,
+        hass,
+        client: evohomeasync2.EvohomeClient,
+        client_v1: evohomeasync.EvohomeClient | None,
+        store: Store[dict[str, Any]],
+        params,
+    ) -> None:
         """Initialize the evohome client and its data structure."""
         self.hass = hass
         self.client = client
@@ -399,7 +416,7 @@ class EvoBroker:
         self.tcs_utc_offset = timedelta(
             minutes=client.locations[loc_idx].timeZone[UTC_OFFSET]
         )
-        self.temps = {}
+        self.temps: dict[str, Any] | None = {}
 
     async def save_auth_tokens(self) -> None:
         """Save access tokens and session IDs to the store for later use."""
@@ -432,13 +449,15 @@ class EvoBroker:
             return
 
         if update_state:  # wait a moment for system to quiesce before updating state
-            self.hass.helpers.event.async_call_later(1, self._update_v2_api_state)
+            async_call_later(self.hass, 1, self._update_v2_api_state)
 
         return result
 
     async def _update_v1_api_temps(self, *args, **kwargs) -> None:
         """Get the latest high-precision temperatures of the default Location."""
 
+        assert self.client_v1
+
         def get_session_id(client_v1) -> str | None:
             user_data = client_v1.user_data if client_v1 else None
             return user_data.get("sessionId") if user_data else None
@@ -514,51 +533,36 @@ class EvoDevice(Entity):
     DHW controller.
     """
 
+    _attr_should_poll = False
+
     def __init__(self, evo_broker, evo_device) -> None:
         """Initialize the evohome entity."""
         self._evo_device = evo_device
         self._evo_broker = evo_broker
         self._evo_tcs = evo_broker.tcs
 
-        self._unique_id = self._name = self._icon = self._precision = None
-        self._supported_features = None
-        self._device_state_attrs = {}
+        self._device_state_attrs: dict[str, Any] = {}
 
     async def async_refresh(self, payload: dict | None = None) -> None:
         """Process any signals."""
         if payload is None:
             self.async_schedule_update_ha_state(force_refresh=True)
             return
-        if payload["unique_id"] != self._unique_id:
+        if payload["unique_id"] != self._attr_unique_id:
             return
-        if payload["service"] in [SVC_SET_ZONE_OVERRIDE, SVC_RESET_ZONE_OVERRIDE]:
+        if payload["service"] in (SVC_SET_ZONE_OVERRIDE, SVC_RESET_ZONE_OVERRIDE):
             await self.async_zone_svc_request(payload["service"], payload["data"])
             return
         await self.async_tcs_svc_request(payload["service"], payload["data"])
 
-    async def async_tcs_svc_request(self, service: dict, data: dict) -> None:
+    async def async_tcs_svc_request(self, service: str, data: dict[str, Any]) -> None:
         """Process a service request (system mode) for a controller."""
         raise NotImplementedError
 
-    async def async_zone_svc_request(self, service: dict, data: dict) -> None:
+    async def async_zone_svc_request(self, service: str, data: dict[str, Any]) -> None:
         """Process a service request (setpoint override) for a zone."""
         raise NotImplementedError
 
-    @property
-    def should_poll(self) -> bool:
-        """Evohome entities should not be polled."""
-        return False
-
-    @property
-    def unique_id(self) -> str | None:
-        """Return a unique ID."""
-        return self._unique_id
-
-    @property
-    def name(self) -> str:
-        """Return the name of the evohome entity."""
-        return self._name
-
     @property
     def extra_state_attributes(self) -> dict[str, Any]:
         """Return the evohome-specific state attributes."""
@@ -572,30 +576,10 @@ class EvoDevice(Entity):
 
         return {"status": convert_dict(status)}
 
-    @property
-    def icon(self) -> str:
-        """Return the icon to use in the frontend UI."""
-        return self._icon
-
-    @property
-    def supported_features(self) -> int:
-        """Get the flag of supported features of the device."""
-        return self._supported_features
-
     async def async_added_to_hass(self) -> None:
         """Run when entity about to be added to hass."""
         async_dispatcher_connect(self.hass, DOMAIN, self.async_refresh)
 
-    @property
-    def precision(self) -> float:
-        """Return the temperature precision to use in the frontend UI."""
-        return self._precision
-
-    @property
-    def temperature_unit(self) -> str:
-        """Return the temperature unit to use in the frontend UI."""
-        return TEMP_CELSIUS
-
 
 class EvoChild(EvoDevice):
     """Base for any evohome child.
@@ -606,8 +590,8 @@ class EvoChild(EvoDevice):
     def __init__(self, evo_broker, evo_device) -> None:
         """Initialize a evohome Controller (hub)."""
         super().__init__(evo_broker, evo_device)
-        self._schedule = {}
-        self._setpoints = {}
+        self._schedule: dict[str, Any] = {}
+        self._setpoints: dict[str, Any] = {}
 
     @property
     def current_temperature(self) -> float | None:
@@ -621,6 +605,8 @@ class EvoChild(EvoDevice):
         if self._evo_device.temperatureStatus["isAvailable"]:
             return self._evo_device.temperatureStatus["temperature"]
 
+        return None
+
     @property
     def setpoints(self) -> dict[str, Any]:
         """Return the current/next setpoints from the schedule.
@@ -653,17 +639,20 @@ class EvoChild(EvoDevice):
             this_sp_day = -1 if sp_idx == -1 else 0
             next_sp_day = 1 if sp_idx + 1 == len(day["Switchpoints"]) else 0
 
-            for key, offset, idx in [
+            for key, offset, idx in (
                 ("this", this_sp_day, sp_idx),
                 ("next", next_sp_day, (sp_idx + 1) * (1 - next_sp_day)),
-            ]:
+            ):
                 sp_date = (day_time + timedelta(days=offset)).strftime("%Y-%m-%d")
                 day = self._schedule["DailySchedules"][(day_of_week + offset) % 7]
                 switchpoint = day["Switchpoints"][idx]
 
+                switchpoint_time_of_day = dt_util.parse_datetime(
+                    f"{sp_date}T{switchpoint['TimeOfDay']}"
+                )
+                assert switchpoint_time_of_day
                 dt_aware = _dt_evo_to_aware(
-                    dt_util.parse_datetime(f"{sp_date}T{switchpoint['TimeOfDay']}"),
-                    self._evo_broker.tcs_utc_offset,
+                    switchpoint_time_of_day, self._evo_broker.tcs_utc_offset
                 )
 
                 self._setpoints[f"{key}_sp_from"] = dt_aware.isoformat()
@@ -692,7 +681,8 @@ class EvoChild(EvoDevice):
     async def async_update(self) -> None:
         """Get the latest state data."""
         next_sp_from = self._setpoints.get("next_sp_from", "2000-01-01T00:00:00+00:00")
-        if dt_util.now() >= dt_util.parse_datetime(next_sp_from):
+        next_sp_from_dt = dt_util.parse_datetime(next_sp_from)
+        if next_sp_from_dt is None or dt_util.now() >= next_sp_from_dt:
             await self._update_schedule()  # no schedule, or it's out-of-date
 
         self._device_state_attrs = {"setpoints": self.setpoints}
