@@ -1,28 +1,30 @@
 """Config flow for Flux LED/MagicLight."""
 from __future__ import annotations
 
-import logging
+import contextlib
 from typing import Any, Final, cast
 
-from flux_led.const import ATTR_ID, ATTR_IPADDR, ATTR_MODEL, ATTR_MODEL_DESCRIPTION
+from flux_led.const import (
+    ATTR_ID,
+    ATTR_IPADDR,
+    ATTR_MODEL,
+    ATTR_MODEL_DESCRIPTION,
+    ATTR_MODEL_INFO,
+    ATTR_VERSION_NUM,
+)
 from flux_led.scanner import FluxLEDDiscovery
 import voluptuous as vol
 
 from homeassistant import config_entries
 from homeassistant.components import dhcp
-from homeassistant.const import CONF_HOST, CONF_MAC, CONF_MODE, CONF_NAME, CONF_PROTOCOL
+from homeassistant.const import CONF_HOST
 from homeassistant.core import callback
-from homeassistant.data_entry_flow import FlowResult
+from homeassistant.data_entry_flow import AbortFlow, FlowResult
 from homeassistant.helpers import device_registry as dr
+from homeassistant.helpers.dispatcher import async_dispatcher_send
 from homeassistant.helpers.typing import DiscoveryInfoType
 
-from . import (
-    async_discover_device,
-    async_discover_devices,
-    async_name_from_discovery,
-    async_update_entry_from_discovery,
-    async_wifi_bulb_for_host,
-)
+from . import async_wifi_bulb_for_host
 from .const import (
     CONF_CUSTOM_EFFECT_COLORS,
     CONF_CUSTOM_EFFECT_SPEED_PCT,
@@ -30,20 +32,26 @@ from .const import (
     DEFAULT_EFFECT_SPEED,
     DISCOVER_SCAN_TIMEOUT,
     DOMAIN,
+    FLUX_LED_DISCOVERY_SIGNAL,
     FLUX_LED_EXCEPTIONS,
     TRANSITION_GRADUAL,
     TRANSITION_JUMP,
     TRANSITION_STROBE,
 )
+from .discovery import (
+    async_discover_device,
+    async_discover_devices,
+    async_name_from_discovery,
+    async_populate_data_from_discovery,
+    async_update_entry_from_discovery,
+)
+from .util import format_as_flux_mac, mac_matches_by_one
 
 CONF_DEVICE: Final = "device"
 
 
-_LOGGER = logging.getLogger(__name__)
-
-
 class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
-    """Handle a config flow for FluxLED/MagicHome Integration."""
+    """Handle a config flow for Magic Home Integration."""
 
     VERSION = 1
 
@@ -51,6 +59,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         """Initialize the config flow."""
         self._discovered_devices: dict[str, FluxLEDDiscovery] = {}
         self._discovered_device: FluxLEDDiscovery | None = None
+        self._allow_update_mac = False
 
     @staticmethod
     @callback
@@ -58,37 +67,12 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         """Get the options flow for the Flux LED component."""
         return OptionsFlow(config_entry)
 
-    async def async_step_import(self, user_input: dict[str, Any]) -> FlowResult:
-        """Handle configuration via YAML import."""
-        _LOGGER.debug("Importing configuration from YAML for flux_led")
-        host = user_input[CONF_HOST]
-        self._async_abort_entries_match({CONF_HOST: host})
-        if mac := user_input[CONF_MAC]:
-            await self.async_set_unique_id(dr.format_mac(mac), raise_on_progress=False)
-            self._abort_if_unique_id_configured(updates={CONF_HOST: host})
-        return self.async_create_entry(
-            title=user_input[CONF_NAME],
-            data={
-                CONF_HOST: host,
-                CONF_NAME: user_input[CONF_NAME],
-                CONF_PROTOCOL: user_input.get(CONF_PROTOCOL),
-            },
-            options={
-                CONF_MODE: user_input[CONF_MODE],
-                CONF_CUSTOM_EFFECT_COLORS: user_input[CONF_CUSTOM_EFFECT_COLORS],
-                CONF_CUSTOM_EFFECT_SPEED_PCT: user_input[CONF_CUSTOM_EFFECT_SPEED_PCT],
-                CONF_CUSTOM_EFFECT_TRANSITION: user_input[
-                    CONF_CUSTOM_EFFECT_TRANSITION
-                ],
-            },
-        )
-
     async def async_step_dhcp(self, discovery_info: dhcp.DhcpServiceInfo) -> FlowResult:
         """Handle discovery via dhcp."""
         self._discovered_device = FluxLEDDiscovery(
             ipaddr=discovery_info.ip,
-            model=discovery_info.hostname,
-            id=discovery_info.macaddress.replace(":", ""),
+            model=None,
+            id=format_as_flux_mac(discovery_info.macaddress),
             model_num=None,
             version_num=None,
             firmware_date=None,
@@ -100,42 +84,83 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         )
         return await self._async_handle_discovery()
 
-    async def async_step_discovery(
+    async def async_step_integration_discovery(
         self, discovery_info: DiscoveryInfoType
     ) -> FlowResult:
-        """Handle discovery."""
+        """Handle integration discovery."""
+        self._allow_update_mac = True
         self._discovered_device = cast(FluxLEDDiscovery, discovery_info)
         return await self._async_handle_discovery()
 
+    async def _async_set_discovered_mac(
+        self, device: FluxLEDDiscovery, allow_update_mac: bool
+    ) -> None:
+        """Set the discovered mac.
+
+        We only allow it to be updated if it comes from udp
+        discovery since the dhcp mac can be one digit off from
+        the udp discovery mac for devices with multiple network interfaces
+        """
+        mac_address = device[ATTR_ID]
+        assert mac_address is not None
+        mac = dr.format_mac(mac_address)
+        await self.async_set_unique_id(mac)
+        for entry in self._async_current_entries(include_ignore=True):
+            if not (
+                entry.data.get(CONF_HOST) == device[ATTR_IPADDR]
+                or (
+                    entry.unique_id
+                    and ":" in entry.unique_id
+                    and mac_matches_by_one(entry.unique_id, mac)
+                )
+            ):
+                continue
+            if entry.source == config_entries.SOURCE_IGNORE:
+                raise AbortFlow("already_configured")
+            if (
+                async_update_entry_from_discovery(
+                    self.hass, entry, device, None, allow_update_mac
+                )
+                or entry.state == config_entries.ConfigEntryState.SETUP_RETRY
+            ):
+                self.hass.async_create_task(
+                    self.hass.config_entries.async_reload(entry.entry_id)
+                )
+            else:
+                async_dispatcher_send(
+                    self.hass,
+                    FLUX_LED_DISCOVERY_SIGNAL.format(entry_id=entry.entry_id),
+                )
+            raise AbortFlow("already_configured")
+
     async def _async_handle_discovery(self) -> FlowResult:
         """Handle any discovery."""
         device = self._discovered_device
         assert device is not None
-        mac_address = device[ATTR_ID]
-        assert mac_address is not None
-        mac = dr.format_mac(mac_address)
+        await self._async_set_discovered_mac(device, self._allow_update_mac)
         host = device[ATTR_IPADDR]
-        await self.async_set_unique_id(mac)
-        self._abort_if_unique_id_configured(updates={CONF_HOST: host})
-        for entry in self._async_current_entries(include_ignore=False):
-            if entry.data[CONF_HOST] == host:
-                if not entry.unique_id:
-                    async_update_entry_from_discovery(self.hass, entry, device)
-                return self.async_abort(reason="already_configured")
         self.context[CONF_HOST] = host
         for progress in self._async_in_progress():
             if progress.get("context", {}).get(CONF_HOST) == host:
                 return self.async_abort(reason="already_in_progress")
         if not device[ATTR_MODEL_DESCRIPTION]:
+            mac_address = device[ATTR_ID]
+            assert mac_address is not None
+            mac = dr.format_mac(mac_address)
             try:
-                device = await self._async_try_connect(
-                    host, device[ATTR_ID], device[ATTR_MODEL]
-                )
+                device = await self._async_try_connect(host, device)
             except FLUX_LED_EXCEPTIONS:
                 return self.async_abort(reason="cannot_connect")
             else:
-                if device[ATTR_MODEL_DESCRIPTION]:
+                discovered_mac = device[ATTR_ID]
+                if device[ATTR_MODEL_DESCRIPTION] or (
+                    discovered_mac is not None
+                    and (formatted_discovered_mac := dr.format_mac(discovered_mac))
+                    and formatted_discovered_mac != mac
+                    and mac_matches_by_one(discovered_mac, mac)
+                ):
                     self._discovered_device = device
+                    await self._async_set_discovered_mac(device, True)
         return await self.async_step_discovery_confirm()
 
     async def async_step_discovery_confirm(
@@ -165,12 +190,11 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         """Create a config entry from a device."""
         self._async_abort_entries_match({CONF_HOST: device[ATTR_IPADDR]})
         name = async_name_from_discovery(device)
+        data: dict[str, Any] = {CONF_HOST: device[ATTR_IPADDR]}
+        async_populate_data_from_discovery(data, data, device)
         return self.async_create_entry(
             title=name,
-            data={
-                CONF_HOST: device[ATTR_IPADDR],
-                CONF_NAME: name,
-            },
+            data=data,
         )
 
     async def async_step_user(
@@ -182,12 +206,11 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             if not (host := user_input[CONF_HOST]):
                 return await self.async_step_pick_device()
             try:
-                device = await self._async_try_connect(host, None, None)
+                device = await self._async_try_connect(host, None)
             except FLUX_LED_EXCEPTIONS:
                 errors["base"] = "cannot_connect"
             else:
-                mac_address = device[ATTR_ID]
-                if mac_address is not None:
+                if (mac_address := device[ATTR_ID]) is not None:
                     await self.async_set_unique_id(
                         dr.format_mac(mac_address), raise_on_progress=False
                     )
@@ -207,7 +230,11 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         if user_input is not None:
             mac = user_input[CONF_DEVICE]
             await self.async_set_unique_id(mac, raise_on_progress=False)
-            return self._async_create_entry_from_device(self._discovered_devices[mac])
+            device = self._discovered_devices[mac]
+            if not device.get(ATTR_MODEL_DESCRIPTION):
+                with contextlib.suppress(*FLUX_LED_EXCEPTIONS):
+                    device = await self._async_try_connect(device[ATTR_IPADDR], device)
+            return self._async_create_entry_from_device(device)
 
         current_unique_ids = self._async_current_ids()
         current_hosts = {
@@ -237,7 +264,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         )
 
     async def _async_try_connect(
-        self, host: str, mac_address: str | None, model: str | None
+        self, host: str, discovery: FluxLEDDiscovery | None
     ) -> FluxLEDDiscovery:
         """Try to connect."""
         self._async_abort_entries_match({CONF_HOST: host})
@@ -250,19 +277,20 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
             # identifying the device as the chip model number
             # AKA `HF-LPB100-ZJ200`
             return device
-        bulb = async_wifi_bulb_for_host(host)
+        bulb = async_wifi_bulb_for_host(host, discovery=device)
+        bulb.discovery = discovery
         try:
             await bulb.async_setup(lambda: None)
         finally:
             await bulb.async_stop()
         return FluxLEDDiscovery(
             ipaddr=host,
-            model=model,
-            id=mac_address,
+            model=discovery[ATTR_MODEL] if discovery else None,
+            id=discovery[ATTR_ID] if discovery else None,
             model_num=bulb.model_num,
-            version_num=bulb.version_num,
+            version_num=discovery[ATTR_VERSION_NUM] if discovery else None,
             firmware_date=None,
-            model_info=None,
+            model_info=discovery[ATTR_MODEL_INFO] if discovery else None,
             model_description=bulb.model_data.description,
             remote_access_enabled=None,
             remote_access_host=None,
