@@ -1,22 +1,27 @@
 """Config flow for generic (IP Camera)."""
 from __future__ import annotations
 
+from collections.abc import Mapping
 import contextlib
 from errno import EHOSTUNREACH, EIO
-from functools import partial
 import io
 import logging
-from types import MappingProxyType
 from typing import Any
 
 import PIL
 from async_timeout import timeout
-import av
 from httpx import HTTPStatusError, RequestError, TimeoutException
 import voluptuous as vol
 import yarl
 
-from homeassistant.components.stream.const import SOURCE_TIMEOUT
+from homeassistant.components.stream import (
+    CONF_RTSP_TRANSPORT,
+    CONF_USE_WALLCLOCK_AS_TIMESTAMPS,
+    HLS_PROVIDER,
+    RTSP_TRANSPORTS,
+    SOURCE_TIMEOUT,
+    create_stream,
+)
 from homeassistant.config_entries import ConfigEntry, ConfigFlow, OptionsFlow
 from homeassistant.const import (
     CONF_AUTHENTICATION,
@@ -27,6 +32,7 @@ from homeassistant.const import (
     HTTP_BASIC_AUTHENTICATION,
     HTTP_DIGEST_AUTHENTICATION,
 )
+from homeassistant.core import HomeAssistant
 from homeassistant.data_entry_flow import FlowResult
 from homeassistant.exceptions import TemplateError
 from homeassistant.helpers import config_validation as cv, template as template_helper
@@ -38,14 +44,11 @@ from .const import (
     CONF_CONTENT_TYPE,
     CONF_FRAMERATE,
     CONF_LIMIT_REFETCH_TO_URL_CHANGE,
-    CONF_RTSP_TRANSPORT,
     CONF_STILL_IMAGE_URL,
     CONF_STREAM_SOURCE,
     DEFAULT_NAME,
     DOMAIN,
-    FFMPEG_OPTION_MAP,
     GET_IMAGE_TIMEOUT,
-    RTSP_TRANSPORTS,
 )
 
 _LOGGER = logging.getLogger(__name__)
@@ -62,8 +65,9 @@ SUPPORTED_IMAGE_TYPES = {"png", "jpeg", "gif", "svg+xml", "webp"}
 
 
 def build_schema(
-    user_input: dict[str, Any] | MappingProxyType[str, Any],
+    user_input: Mapping[str, Any],
     is_options_flow: bool = False,
+    show_advanced_options=False,
 ):
     """Create schema for camera config setup."""
     spec = {
@@ -94,7 +98,7 @@ def build_schema(
         vol.Required(
             CONF_FRAMERATE,
             description={"suggested_value": user_input.get(CONF_FRAMERATE, 2)},
-        ): int,
+        ): vol.All(vol.Range(min=0, min_included=False), cv.positive_float),
         vol.Required(
             CONF_VERIFY_SSL, default=user_input.get(CONF_VERIFY_SSL, True)
         ): bool,
@@ -106,10 +110,17 @@ def build_schema(
                 default=user_input.get(CONF_LIMIT_REFETCH_TO_URL_CHANGE, False),
             )
         ] = bool
+        if show_advanced_options:
+            spec[
+                vol.Required(
+                    CONF_USE_WALLCLOCK_AS_TIMESTAMPS,
+                    default=user_input.get(CONF_USE_WALLCLOCK_AS_TIMESTAMPS, False),
+                )
+            ] = bool
     return vol.Schema(spec)
 
 
-def get_image_type(image):
+def get_image_type(image: bytes) -> str | None:
     """Get the format of downloaded bytes that could be an image."""
     fmt = None
     imagefile = io.BytesIO(image)
@@ -125,20 +136,27 @@ def get_image_type(image):
     return fmt
 
 
-async def async_test_still(hass, info) -> tuple[dict[str, str], str | None]:
+async def async_test_still(
+    hass: HomeAssistant, info: Mapping[str, Any]
+) -> tuple[dict[str, str], str | None]:
     """Verify that the still image is valid before we create an entity."""
     fmt = None
     if not (url := info.get(CONF_STILL_IMAGE_URL)):
         return {}, info.get(CONF_CONTENT_TYPE, "image/jpeg")
-    if not isinstance(url, template_helper.Template) and url:
-        url = cv.template(url)
-        url.hass = hass
     try:
+        if not isinstance(url, template_helper.Template):
+            url = template_helper.Template(url, hass)
         url = url.async_render(parse_result=False)
     except TemplateError as err:
         _LOGGER.warning("Problem rendering template %s: %s", url, err)
         return {CONF_STILL_IMAGE_URL: "template_error"}, None
-    verify_ssl = info.get(CONF_VERIFY_SSL)
+    try:
+        yarl_url = yarl.URL(url)
+    except ValueError:
+        return {CONF_STILL_IMAGE_URL: "malformed_url"}, None
+    if not yarl_url.is_absolute():
+        return {CONF_STILL_IMAGE_URL: "relative_url"}, None
+    verify_ssl = info[CONF_VERIFY_SSL]
     auth = generate_auth(info)
     try:
         async_client = get_async_client(hass, verify_ssl=verify_ssl)
@@ -168,49 +186,70 @@ async def async_test_still(hass, info) -> tuple[dict[str, str], str | None]:
     return {}, f"image/{fmt}"
 
 
-def slug_url(url) -> str | None:
+def slug(
+    hass: HomeAssistant, template: str | template_helper.Template | None
+) -> str | None:
     """Convert a camera url into a string suitable for a camera name."""
-    if not url:
+    if not template:
         return None
-    return slugify(yarl.URL(url).host)
+    if not isinstance(template, template_helper.Template):
+        template = template_helper.Template(template, hass)
+    try:
+        url = template.async_render(parse_result=False)
+        return slugify(yarl.URL(url).host)
+    except TemplateError as err:
+        _LOGGER.error("Syntax error in '%s': %s", template.template, err)
+    except (ValueError, TypeError) as err:
+        _LOGGER.error("Syntax error in '%s': %s", url, err)
+    return None
 
 
-async def async_test_stream(hass, info) -> dict[str, str]:
+async def async_test_stream(
+    hass: HomeAssistant, info: Mapping[str, Any]
+) -> dict[str, str]:
     """Verify that the stream is valid before we create an entity."""
     if not (stream_source := info.get(CONF_STREAM_SOURCE)):
         return {}
+    # Import from stream.worker as stream cannot reexport from worker
+    # without forcing the av dependency on default_config
+    # pylint: disable=import-outside-toplevel
+    from homeassistant.components.stream.worker import StreamWorkerError
+
+    if not isinstance(stream_source, template_helper.Template):
+        stream_source = template_helper.Template(stream_source, hass)
     try:
-        # For RTSP streams, prefer TCP. This code is duplicated from
-        # homeassistant.components.stream.__init__.py:create_stream()
-        # It may be possible & better to call create_stream() directly.
-        stream_options: dict[str, str] = {}
-        if isinstance(stream_source, str) and stream_source[:7] == "rtsp://":
-            stream_options = {
-                "rtsp_flags": "prefer_tcp",
-                "stimeout": "5000000",
-            }
-        if rtsp_transport := info.get(CONF_RTSP_TRANSPORT):
-            stream_options[FFMPEG_OPTION_MAP[CONF_RTSP_TRANSPORT]] = rtsp_transport
-        _LOGGER.debug("Attempting to open stream %s", stream_source)
-        container = await hass.async_add_executor_job(
-            partial(
-                av.open,
-                stream_source,
-                options=stream_options,
-                timeout=SOURCE_TIMEOUT,
-            )
-        )
-        _ = container.streams.video[0]
-    except (av.error.FileNotFoundError):  # pylint: disable=c-extension-no-member
-        return {CONF_STREAM_SOURCE: "stream_file_not_found"}
-    except (av.error.HTTPNotFoundError):  # pylint: disable=c-extension-no-member
-        return {CONF_STREAM_SOURCE: "stream_http_not_found"}
-    except (av.error.TimeoutError):  # pylint: disable=c-extension-no-member
-        return {CONF_STREAM_SOURCE: "timeout"}
-    except av.error.HTTPUnauthorizedError:  # pylint: disable=c-extension-no-member
-        return {CONF_STREAM_SOURCE: "stream_unauthorised"}
-    except (KeyError, IndexError):
-        return {CONF_STREAM_SOURCE: "stream_no_video"}
+        stream_source = stream_source.async_render(parse_result=False)
+    except TemplateError as err:
+        _LOGGER.warning("Problem rendering template %s: %s", stream_source, err)
+        return {CONF_STREAM_SOURCE: "template_error"}
+    stream_options: dict[str, str | bool | float] = {}
+    if rtsp_transport := info.get(CONF_RTSP_TRANSPORT):
+        stream_options[CONF_RTSP_TRANSPORT] = rtsp_transport
+    if info.get(CONF_USE_WALLCLOCK_AS_TIMESTAMPS):
+        stream_options[CONF_USE_WALLCLOCK_AS_TIMESTAMPS] = True
+
+    try:
+        url = yarl.URL(stream_source)
+    except ValueError:
+        return {CONF_STREAM_SOURCE: "malformed_url"}
+    if not url.is_absolute():
+        return {CONF_STREAM_SOURCE: "relative_url"}
+    if not url.user and not url.password:
+        username = info.get(CONF_USERNAME)
+        password = info.get(CONF_PASSWORD)
+        if username and password:
+            url = url.with_user(username).with_password(password)
+            stream_source = str(url)
+    try:
+        stream = create_stream(hass, stream_source, stream_options, "test_stream")
+        hls_provider = stream.add_provider(HLS_PROVIDER)
+        await stream.start()
+        if not await hls_provider.part_recv(timeout=SOURCE_TIMEOUT):
+            hass.async_create_task(stream.stop())
+            return {CONF_STREAM_SOURCE: "timeout"}
+        await stream.stop()
+    except StreamWorkerError as err:
+        return {CONF_STREAM_SOURCE: str(err)}
     except PermissionError:
         return {CONF_STREAM_SOURCE: "stream_not_permitted"}
     except OSError as err:
@@ -227,7 +266,7 @@ class GenericIPCamConfigFlow(ConfigFlow, domain=DOMAIN):
 
     VERSION = 1
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize Generic ConfigFlow."""
         self.cached_user_input: dict[str, Any] = {}
         self.cached_title = ""
@@ -239,7 +278,7 @@ class GenericIPCamConfigFlow(ConfigFlow, domain=DOMAIN):
         """Get the options flow for this handler."""
         return GenericOptionsFlowHandler(config_entry)
 
-    def check_for_existing(self, options):
+    def check_for_existing(self, options: dict[str, Any]) -> bool:
         """Check whether an existing entry is using the same URLs."""
         return any(
             entry.options.get(CONF_STILL_IMAGE_URL) == options.get(CONF_STILL_IMAGE_URL)
@@ -252,6 +291,7 @@ class GenericIPCamConfigFlow(ConfigFlow, domain=DOMAIN):
     ) -> FlowResult:
         """Handle the start of the config flow."""
         errors = {}
+        hass = self.hass
         if user_input:
             # Secondary validation because serialised vol can't seem to handle this complexity:
             if not user_input.get(CONF_STILL_IMAGE_URL) and not user_input.get(
@@ -259,22 +299,22 @@ class GenericIPCamConfigFlow(ConfigFlow, domain=DOMAIN):
             ):
                 errors["base"] = "no_still_image_or_stream_url"
             else:
-                errors, still_format = await async_test_still(self.hass, user_input)
-                errors = errors | await async_test_stream(self.hass, user_input)
-                still_url = user_input.get(CONF_STILL_IMAGE_URL)
-                stream_url = user_input.get(CONF_STREAM_SOURCE)
-                name = slug_url(still_url) or slug_url(stream_url) or DEFAULT_NAME
-
+                errors, still_format = await async_test_still(hass, user_input)
+                errors = errors | await async_test_stream(hass, user_input)
                 if not errors:
                     user_input[CONF_CONTENT_TYPE] = still_format
                     user_input[CONF_LIMIT_REFETCH_TO_URL_CHANGE] = False
+                    still_url = user_input.get(CONF_STILL_IMAGE_URL)
+                    stream_url = user_input.get(CONF_STREAM_SOURCE)
+                    name = (
+                        slug(hass, still_url) or slug(hass, stream_url) or DEFAULT_NAME
+                    )
                     if still_url is None:
                         # If user didn't specify a still image URL,
                         # The automatically generated still image that stream generates
                         # is always jpeg
                         user_input[CONF_CONTENT_TYPE] = "image/jpeg"
 
-                    await self.async_set_unique_id(self.flow_id)
                     return self.async_create_entry(
                         title=name, data={}, options=user_input
                     )
@@ -287,7 +327,7 @@ class GenericIPCamConfigFlow(ConfigFlow, domain=DOMAIN):
             errors=errors,
         )
 
-    async def async_step_import(self, import_config) -> FlowResult:
+    async def async_step_import(self, import_config: dict[str, Any]) -> FlowResult:
         """Handle config import from yaml."""
         # abort if we've already got this one.
         if self.check_for_existing(import_config):
@@ -296,13 +336,14 @@ class GenericIPCamConfigFlow(ConfigFlow, domain=DOMAIN):
         still_url = import_config.get(CONF_STILL_IMAGE_URL)
         stream_url = import_config.get(CONF_STREAM_SOURCE)
         name = import_config.get(
-            CONF_NAME, slug_url(still_url) or slug_url(stream_url) or DEFAULT_NAME
+            CONF_NAME,
+            slug(self.hass, still_url) or slug(self.hass, stream_url) or DEFAULT_NAME,
         )
+
         if CONF_LIMIT_REFETCH_TO_URL_CHANGE not in import_config:
             import_config[CONF_LIMIT_REFETCH_TO_URL_CHANGE] = False
         still_format = import_config.get(CONF_CONTENT_TYPE, "image/jpeg")
         import_config[CONF_CONTENT_TYPE] = still_format
-        await self.async_set_unique_id(self.flow_id)
         return self.async_create_entry(title=name, data={}, options=import_config)
 
 
@@ -320,16 +361,17 @@ class GenericOptionsFlowHandler(OptionsFlow):
     ) -> FlowResult:
         """Manage Generic IP Camera options."""
         errors: dict[str, str] = {}
+        hass = self.hass
 
         if user_input is not None:
             errors, still_format = await async_test_still(
-                self.hass, self.config_entry.options | user_input
+                hass, self.config_entry.options | user_input
             )
-            errors = errors | await async_test_stream(self.hass, user_input)
+            errors = errors | await async_test_stream(hass, user_input)
             still_url = user_input.get(CONF_STILL_IMAGE_URL)
             stream_url = user_input.get(CONF_STREAM_SOURCE)
             if not errors:
-                title = slug_url(still_url) or slug_url(stream_url) or DEFAULT_NAME
+                title = slug(hass, still_url) or slug(hass, stream_url) or DEFAULT_NAME
                 if still_url is None:
                     # If user didn't specify a still image URL,
                     # The automatically generated still image that stream generates
@@ -348,6 +390,12 @@ class GenericOptionsFlowHandler(OptionsFlow):
                     ],
                     CONF_FRAMERATE: user_input[CONF_FRAMERATE],
                     CONF_VERIFY_SSL: user_input[CONF_VERIFY_SSL],
+                    CONF_USE_WALLCLOCK_AS_TIMESTAMPS: user_input.get(
+                        CONF_USE_WALLCLOCK_AS_TIMESTAMPS,
+                        self.config_entry.options.get(
+                            CONF_USE_WALLCLOCK_AS_TIMESTAMPS, False
+                        ),
+                    ),
                 }
                 return self.async_create_entry(
                     title=title,
@@ -355,6 +403,10 @@ class GenericOptionsFlowHandler(OptionsFlow):
                 )
         return self.async_show_form(
             step_id="init",
-            data_schema=build_schema(user_input or self.config_entry.options, True),
+            data_schema=build_schema(
+                user_input or self.config_entry.options,
+                True,
+                self.show_advanced_options,
+            ),
             errors=errors,
         )
