@@ -1,19 +1,24 @@
 """Provides a light for Home Connect."""
 import logging
 from math import ceil
+from typing import Any
 
 from homeconnect.api import HomeConnectError
 
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
     ATTR_HS_COLOR,
-    SUPPORT_BRIGHTNESS,
-    SUPPORT_COLOR,
+    ColorMode,
     LightEntity,
 )
+from homeassistant.config_entries import ConfigEntry
+from homeassistant.const import CONF_ENTITIES
+from homeassistant.core import HomeAssistant
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 import homeassistant.util.color as color_util
 
 from .const import (
+    ATTR_VALUE,
     BSH_AMBIENT_LIGHT_BRIGHTNESS,
     BSH_AMBIENT_LIGHT_COLOR,
     BSH_AMBIENT_LIGHT_COLOR_CUSTOM_COLOR,
@@ -28,7 +33,11 @@ from .entity import HomeConnectEntity
 _LOGGER = logging.getLogger(__name__)
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the Home Connect light."""
 
     def get_entities():
@@ -36,7 +45,7 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
         entities = []
         hc_api = hass.data[DOMAIN][config_entry.entry_id]
         for device_dict in hc_api.devices:
-            entity_dicts = device_dict.get("entities", {}).get("light", [])
+            entity_dicts = device_dict.get(CONF_ENTITIES, {}).get("light", [])
             entity_list = [HomeConnectLight(**d) for d in entity_dicts]
             entities += entity_list
         return entities
@@ -59,11 +68,15 @@ class HomeConnectLight(HomeConnectEntity, LightEntity):
             self._key = BSH_AMBIENT_LIGHT_ENABLED
             self._custom_color_key = BSH_AMBIENT_LIGHT_CUSTOM_COLOR
             self._color_key = BSH_AMBIENT_LIGHT_COLOR
+            self._attr_color_mode = ColorMode.HS
+            self._attr_supported_color_modes = {ColorMode.HS}
         else:
             self._brightness_key = COOKING_LIGHTING_BRIGHTNESS
             self._key = COOKING_LIGHTING
             self._custom_color_key = None
             self._color_key = None
+            self._attr_color_mode = ColorMode.BRIGHTNESS
+            self._attr_supported_color_modes = {ColorMode.BRIGHTNESS}
 
     @property
     def is_on(self):
@@ -80,16 +93,17 @@ class HomeConnectLight(HomeConnectEntity, LightEntity):
         """Return the color property."""
         return self._hs_color
 
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        if self._ambient:
-            return SUPPORT_BRIGHTNESS | SUPPORT_COLOR
-        return SUPPORT_BRIGHTNESS
-
-    async def async_turn_on(self, **kwargs):
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Switch the light on, change brightness, change color."""
         if self._ambient:
+            _LOGGER.debug("Switching ambient light on for: %s", self.name)
+            try:
+                await self.hass.async_add_executor_job(
+                    self.device.appliance.set_setting, self._key, True
+                )
+            except HomeConnectError as err:
+                _LOGGER.error("Error while trying to turn on ambient light: %s", err)
+                return
             if ATTR_BRIGHTNESS in kwargs or ATTR_HS_COLOR in kwargs:
                 try:
                     await self.hass.async_add_executor_job(
@@ -104,10 +118,12 @@ class HomeConnectLight(HomeConnectEntity, LightEntity):
                     if ATTR_BRIGHTNESS in kwargs:
                         brightness = 10 + ceil(kwargs[ATTR_BRIGHTNESS] / 255 * 90)
 
-                    hs_color = kwargs.get(ATTR_HS_COLOR, default=self._hs_color)
+                    hs_color = kwargs.get(ATTR_HS_COLOR, self._hs_color)
 
                     if hs_color is not None:
-                        rgb = color_util.color_hsv_to_RGB(*hs_color, brightness)
+                        rgb = color_util.color_hsv_to_RGB(
+                            hs_color[0], hs_color[1], brightness
+                        )
                         hex_val = color_util.color_rgb_to_hex(rgb[0], rgb[1], rgb[2])
                         try:
                             await self.hass.async_add_executor_job(
@@ -119,27 +135,13 @@ class HomeConnectLight(HomeConnectEntity, LightEntity):
                             _LOGGER.error(
                                 "Error while trying setting the color: %s", err
                             )
-            else:
-                _LOGGER.debug("Switching ambient light on for: %s", self.name)
-                try:
-                    await self.hass.async_add_executor_job(
-                        self.device.appliance.set_setting,
-                        self._key,
-                        True,
-                    )
-                except HomeConnectError as err:
-                    _LOGGER.error(
-                        "Error while trying to turn on ambient light: %s", err
-                    )
 
         elif ATTR_BRIGHTNESS in kwargs:
             _LOGGER.debug("Changing brightness for: %s", self.name)
             brightness = 10 + ceil(kwargs[ATTR_BRIGHTNESS] / 255 * 90)
             try:
                 await self.hass.async_add_executor_job(
-                    self.device.appliance.set_setting,
-                    self._brightness_key,
-                    brightness,
+                    self.device.appliance.set_setting, self._brightness_key, brightness
                 )
             except HomeConnectError as err:
                 _LOGGER.error("Error while trying set the brightness: %s", err)
@@ -147,33 +149,29 @@ class HomeConnectLight(HomeConnectEntity, LightEntity):
             _LOGGER.debug("Switching light on for: %s", self.name)
             try:
                 await self.hass.async_add_executor_job(
-                    self.device.appliance.set_setting,
-                    self._key,
-                    True,
+                    self.device.appliance.set_setting, self._key, True
                 )
             except HomeConnectError as err:
                 _LOGGER.error("Error while trying to turn on light: %s", err)
 
         self.async_entity_update()
 
-    async def async_turn_off(self, **kwargs):
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Switch the light off."""
         _LOGGER.debug("Switching light off for: %s", self.name)
         try:
             await self.hass.async_add_executor_job(
-                self.device.appliance.set_setting,
-                self._key,
-                False,
+                self.device.appliance.set_setting, self._key, False
             )
         except HomeConnectError as err:
             _LOGGER.error("Error while trying to turn off light: %s", err)
         self.async_entity_update()
 
-    async def async_update(self):
+    async def async_update(self) -> None:
         """Update the light's status."""
-        if self.device.appliance.status.get(self._key, {}).get("value") is True:
+        if self.device.appliance.status.get(self._key, {}).get(ATTR_VALUE) is True:
             self._state = True
-        elif self.device.appliance.status.get(self._key, {}).get("value") is False:
+        elif self.device.appliance.status.get(self._key, {}).get(ATTR_VALUE) is False:
             self._state = False
         else:
             self._state = None
@@ -187,7 +185,7 @@ class HomeConnectLight(HomeConnectEntity, LightEntity):
                 self._hs_color = None
                 self._brightness = None
             else:
-                colorvalue = color.get("value")[1:]
+                colorvalue = color.get(ATTR_VALUE)[1:]
                 rgb = color_util.rgb_hex_to_rgb_list(colorvalue)
                 hsv = color_util.color_RGB_to_hsv(rgb[0], rgb[1], rgb[2])
                 self._hs_color = [hsv[0], hsv[1]]
@@ -199,5 +197,5 @@ class HomeConnectLight(HomeConnectEntity, LightEntity):
             if brightness is None:
                 self._brightness = None
             else:
-                self._brightness = ceil((brightness.get("value") - 10) * 255 / 90)
+                self._brightness = ceil((brightness.get(ATTR_VALUE) - 10) * 255 / 90)
             _LOGGER.debug("Updated, new brightness: %s", self._brightness)
