@@ -3,7 +3,7 @@ import asyncio
 from datetime import timedelta
 import logging
 
-from haffmpeg.core import HAFFmpeg
+from haffmpeg.core import FFMPEG_STDERR, HAFFmpeg
 from pyhap.camera import (
     VIDEO_CODEC_PARAM_LEVEL_TYPES,
     VIDEO_CODEC_PARAM_PROFILE_ID_TYPES,
@@ -11,14 +11,14 @@ from pyhap.camera import (
 )
 from pyhap.const import CATEGORY_CAMERA
 
-from homeassistant.components.ffmpeg import DATA_FFMPEG
+from homeassistant.components import camera
+from homeassistant.components.ffmpeg import get_ffmpeg_manager
 from homeassistant.const import STATE_ON
-from homeassistant.core import callback
+from homeassistant.core import Event, callback
 from homeassistant.helpers.event import (
     async_track_state_change_event,
     async_track_time_interval,
 )
-from homeassistant.util import get_local_ip
 
 from .accessories import TYPES, HomeAccessory
 from .const import (
@@ -56,8 +56,7 @@ from .const import (
     SERV_SPEAKER,
     SERV_STATELESS_PROGRAMMABLE_SWITCH,
 )
-from .img_util import scale_jpeg_camera_image
-from .util import pid_is_alive
+from .util import pid_is_alive, state_changed_event_is_same_state
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -109,11 +108,13 @@ RESOLUTIONS = [
     (1280, 720),
     (1280, 960),
     (1920, 1080),
+    (1600, 1200),
 ]
 
 VIDEO_PROFILE_NAMES = ["baseline", "main", "high"]
 
 FFMPEG_WATCH_INTERVAL = timedelta(seconds=5)
+FFMPEG_LOGGER = "ffmpeg_logger"
 FFMPEG_WATCHER = "ffmpeg_watcher"
 FFMPEG_PID = "ffmpeg_pid"
 SESSION_ID = "session_id"
@@ -139,10 +140,10 @@ class Camera(HomeAccessory, PyhapCamera):
 
     def __init__(self, hass, driver, name, entity_id, aid, config):
         """Initialize a Camera accessory object."""
-        self._ffmpeg = hass.data[DATA_FFMPEG]
-        for config_key in CONFIG_DEFAULTS:
+        self._ffmpeg = get_ffmpeg_manager(hass)
+        for config_key, conf in CONFIG_DEFAULTS.items():
             if config_key not in config:
-                config[config_key] = CONFIG_DEFAULTS[config_key]
+                config[config_key] = conf
 
         max_fps = config[CONF_MAX_FPS]
         max_width = config[CONF_MAX_WIDTH]
@@ -179,7 +180,7 @@ class Camera(HomeAccessory, PyhapCamera):
             ]
         }
 
-        stream_address = config.get(CONF_STREAM_ADDRESS, get_local_ip())
+        stream_address = config.get(CONF_STREAM_ADDRESS, driver.state.address)
 
         options = {
             "video": video_options,
@@ -220,46 +221,54 @@ class Camera(HomeAccessory, PyhapCamera):
                 serv_doorbell = self.add_preload_service(SERV_DOORBELL)
                 self.set_primary_service(serv_doorbell)
                 self._char_doorbell_detected = serv_doorbell.configure_char(
-                    CHAR_PROGRAMMABLE_SWITCH_EVENT, value=0,
+                    CHAR_PROGRAMMABLE_SWITCH_EVENT,
+                    value=0,
                 )
                 serv_stateless_switch = self.add_preload_service(
                     SERV_STATELESS_PROGRAMMABLE_SWITCH
                 )
-                self._char_doorbell_detected_switch = serv_stateless_switch.configure_char(
-                    CHAR_PROGRAMMABLE_SWITCH_EVENT,
-                    value=0,
-                    valid_values={"SinglePress": DOORBELL_SINGLE_PRESS},
+                self._char_doorbell_detected_switch = (
+                    serv_stateless_switch.configure_char(
+                        CHAR_PROGRAMMABLE_SWITCH_EVENT,
+                        value=0,
+                        valid_values={"SinglePress": DOORBELL_SINGLE_PRESS},
+                    )
                 )
                 serv_speaker = self.add_preload_service(SERV_SPEAKER)
                 serv_speaker.configure_char(CHAR_MUTE, value=0)
 
                 self._async_update_doorbell_state(state)
 
-    async def run_handler(self):
+    async def run(self):
         """Handle accessory driver started event.
 
         Run inside the Home Assistant event loop.
         """
         if self._char_motion_detected:
-            async_track_state_change_event(
-                self.hass,
-                [self.linked_motion_sensor],
-                self._async_update_motion_state_event,
+            self._subscriptions.append(
+                async_track_state_change_event(
+                    self.hass,
+                    [self.linked_motion_sensor],
+                    self._async_update_motion_state_event,
+                )
             )
 
         if self._char_doorbell_detected:
-            async_track_state_change_event(
-                self.hass,
-                [self.linked_doorbell_sensor],
-                self._async_update_doorbell_state_event,
+            self._subscriptions.append(
+                async_track_state_change_event(
+                    self.hass,
+                    [self.linked_doorbell_sensor],
+                    self._async_update_doorbell_state_event,
+                )
             )
 
-        await super().run_handler()
+        await super().run()
 
     @callback
-    def _async_update_motion_state_event(self, event):
+    def _async_update_motion_state_event(self, event: Event) -> None:
         """Handle state change event listener callback."""
-        self._async_update_motion_state(event.data.get("new_state"))
+        if not state_changed_event_is_same_state(event):
+            self._async_update_motion_state(event.data.get("new_state"))
 
     @callback
     def _async_update_motion_state(self, new_state):
@@ -280,9 +289,10 @@ class Camera(HomeAccessory, PyhapCamera):
         )
 
     @callback
-    def _async_update_doorbell_state_event(self, event):
+    def _async_update_doorbell_state_event(self, event: Event) -> None:
         """Handle state change event listener callback."""
-        self._async_update_doorbell_state(event.data.get("new_state"))
+        if not state_changed_event_is_same_state(event):
+            self._async_update_doorbell_state(event.data.get("new_state"))
 
     @callback
     def _async_update_doorbell_state(self, new_state):
@@ -307,19 +317,16 @@ class Camera(HomeAccessory, PyhapCamera):
 
     async def _async_get_stream_source(self):
         """Find the camera stream source url."""
-        stream_source = self.config.get(CONF_STREAM_SOURCE)
-        if stream_source:
+        if stream_source := self.config.get(CONF_STREAM_SOURCE):
             return stream_source
         try:
-            stream_source = await self.hass.components.camera.async_get_stream_source(
-                self.entity_id
+            stream_source = await camera.async_get_stream_source(
+                self.hass, self.entity_id
             )
         except Exception:  # pylint: disable=broad-except
             _LOGGER.exception(
                 "Failed to get stream source - this could be a transient error or your camera might not be compatible with HomeKit yet"
             )
-        if stream_source:
-            self.config[CONF_STREAM_SOURCE] = stream_source
         return stream_source
 
     async def start_stream(self, session_info, stream_config):
@@ -329,8 +336,7 @@ class Camera(HomeAccessory, PyhapCamera):
             session_info["id"],
             stream_config,
         )
-        input_source = await self._async_get_stream_source()
-        if not input_source:
+        if not (input_source := await self._async_get_stream_source()):
             _LOGGER.error("Camera has no stream source")
             return False
         if "-i " not in input_source:
@@ -366,9 +372,14 @@ class Camera(HomeAccessory, PyhapCamera):
         if self.config[CONF_SUPPORT_AUDIO]:
             output = output + " " + AUDIO_OUTPUT.format(**output_vars)
         _LOGGER.debug("FFmpeg output settings: %s", output)
-        stream = HAFFmpeg(self._ffmpeg.binary, loop=self.driver.loop)
+        stream = HAFFmpeg(self._ffmpeg.binary)
         opened = await stream.open(
-            cmd=[], input_source=input_source, output=output, stdout_pipe=False
+            cmd=[],
+            input_source=input_source,
+            output=output,
+            extra_cmd="-hide_banner -nostats",
+            stderr_pipe=True,
+            stdout_pipe=False,
         )
         if not opened:
             _LOGGER.error("Failed to open ffmpeg stream")
@@ -383,15 +394,32 @@ class Camera(HomeAccessory, PyhapCamera):
         session_info["stream"] = stream
         session_info[FFMPEG_PID] = stream.process.pid
 
+        stderr_reader = await stream.get_reader(source=FFMPEG_STDERR)
+
         async def watch_session(_):
             await self._async_ffmpeg_watch(session_info["id"])
 
+        session_info[FFMPEG_LOGGER] = asyncio.create_task(
+            self._async_log_stderr_stream(stderr_reader)
+        )
         session_info[FFMPEG_WATCHER] = async_track_time_interval(
-            self.hass, watch_session, FFMPEG_WATCH_INTERVAL,
+            self.hass,
+            watch_session,
+            FFMPEG_WATCH_INTERVAL,
         )
 
         return await self._async_ffmpeg_watch(session_info["id"])
 
+    async def _async_log_stderr_stream(self, stderr_reader):
+        """Log output from ffmpeg."""
+        _LOGGER.debug("%s: ffmpeg: started", self.display_name)
+        while True:
+            line = await stderr_reader.readline()
+            if line == b"":
+                return
+
+            _LOGGER.debug("%s: ffmpeg: %s", self.display_name, line.rstrip())
+
     async def _async_ffmpeg_watch(self, session_id):
         """Check to make sure ffmpeg is still running and cleanup if not."""
         ffmpeg_pid = self.sessions[session_id][FFMPEG_PID]
@@ -409,12 +437,18 @@ class Camera(HomeAccessory, PyhapCamera):
         if FFMPEG_WATCHER not in self.sessions[session_id]:
             return
         self.sessions[session_id].pop(FFMPEG_WATCHER)()
+        self.sessions[session_id].pop(FFMPEG_LOGGER).cancel()
+
+    async def stop(self):
+        """Stop any streams when the accessory is stopped."""
+        for session_info in self.sessions.values():
+            asyncio.create_task(self.stop_stream(session_info))
+        await super().stop()
 
     async def stop_stream(self, session_info):
         """Stop the stream for the given ``session_id``."""
         session_id = session_info["id"]
-        stream = session_info.get("stream")
-        if not stream:
+        if not (stream := session_info.get("stream")):
             _LOGGER.debug("No stream for session ID %s", session_id)
             return
 
@@ -424,7 +458,7 @@ class Camera(HomeAccessory, PyhapCamera):
             _LOGGER.info("[%s] Stream already stopped", session_id)
             return True
 
-        for shutdown_method in ["close", "kill"]:
+        for shutdown_method in ("close", "kill"):
             _LOGGER.info("[%s] %s stream", session_id, shutdown_method)
             try:
                 await getattr(stream, shutdown_method)()
@@ -438,13 +472,12 @@ class Camera(HomeAccessory, PyhapCamera):
         """Reconfigure the stream so that it uses the given ``stream_config``."""
         return True
 
-    def get_snapshot(self, image_size):
+    async def async_get_snapshot(self, image_size):
         """Return a jpeg of a snapshot from the camera."""
-        return scale_jpeg_camera_image(
-            asyncio.run_coroutine_threadsafe(
-                self.hass.components.camera.async_get_image(self.entity_id),
-                self.hass.loop,
-            ).result(),
-            image_size["image-width"],
-            image_size["image-height"],
+        image = await camera.async_get_image(
+            self.hass,
+            self.entity_id,
+            width=image_size["image-width"],
+            height=image_size["image-height"],
         )
+        return image.content
