@@ -8,11 +8,12 @@ from __future__ import annotations
 
 from aiohomekit.model.characteristics import Characteristic, CharacteristicsTypes
 
-from homeassistant.components.number import NumberEntity, NumberEntityDescription
-from homeassistant.components.number.const import (
+from homeassistant.components.number import (
     DEFAULT_MAX_VALUE,
     DEFAULT_MIN_VALUE,
     DEFAULT_STEP,
+    NumberEntity,
+    NumberEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
 from homeassistant.core import HomeAssistant, callback
@@ -20,8 +21,9 @@ from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.typing import ConfigType
 
-from . import KNOWN_DEVICES, CharacteristicEntity
+from . import KNOWN_DEVICES
 from .connection import HKDevice
+from .entity import CharacteristicEntity
 
 NUMBER_ENTITIES: dict[str, NumberEntityDescription] = {
     CharacteristicsTypes.VENDOR_VOCOLINC_HUMIDIFIER_SPRAY_LEVEL: NumberEntityDescription(
@@ -58,7 +60,7 @@ async def async_setup_entry(
 ) -> None:
     """Set up Homekit numbers."""
     hkid = config_entry.data["AccessoryPairingID"]
-    conn = hass.data[KNOWN_DEVICES][hkid]
+    conn: HKDevice = hass.data[KNOWN_DEVICES][hkid]
 
     @callback
     def async_add_characteristic(char: Characteristic) -> bool:
@@ -67,8 +69,6 @@ async def async_setup_entry(
 
         if description := NUMBER_ENTITIES.get(char.type):
             entities.append(HomeKitNumber(conn, info, char, description))
-        elif entity_type := NUMBER_ENTITY_CLASSES.get(char.type):
-            entities.append(entity_type(conn, info, char))
         else:
             return False
 
@@ -92,60 +92,16 @@ class HomeKitNumber(CharacteristicEntity, NumberEntity):
         self.entity_description = description
         super().__init__(conn, info, char)
 
-    @property
-    def name(self) -> str | None:
-        """Return the name of the device if any."""
-        if prefix := super().name:
-            return f"{prefix} {self.entity_description.name}"
-        return self.entity_description.name
-
-    def get_characteristic_types(self) -> list[str]:
-        """Define the homekit characteristics the entity is tracking."""
-        return [self._char.type]
-
-    @property
-    def native_min_value(self) -> float:
-        """Return the minimum value."""
-        return self._char.minValue or DEFAULT_MIN_VALUE
-
-    @property
-    def native_max_value(self) -> float:
-        """Return the maximum value."""
-        return self._char.maxValue or DEFAULT_MAX_VALUE
-
-    @property
-    def native_step(self) -> float:
-        """Return the increment/decrement step."""
-        return self._char.minStep or DEFAULT_STEP
-
-    @property
-    def native_value(self) -> float:
-        """Return the current characteristic value."""
-        return self._char.value
-
-    async def async_set_native_value(self, value: float) -> None:
-        """Set the characteristic to this value."""
-        await self.async_put_characteristics(
-            {
-                self._char.type: value,
-            }
-        )
-
-
-class HomeKitEcobeeFanModeNumber(CharacteristicEntity, NumberEntity):
-    """Representation of a Number control for Ecobee Fan Mode request."""
-
-    def get_characteristic_types(self) -> list[str]:
-        """Define the homekit characteristics the entity is tracking."""
-        return [self._char.type]
-
     @property
     def name(self) -> str:
         """Return the name of the device if any."""
-        prefix = ""
-        if name := super().name:
-            prefix = name
-        return f"{prefix} Fan Mode"
+        if name := self.accessory.name:
+            return f"{name} {self.entity_description.name}"
+        return f"{self.entity_description.name}"
+
+    def get_characteristic_types(self) -> list[str]:
+        """Define the homekit characteristics the entity is tracking."""
+        return [self._char.type]
 
     @property
     def native_min_value(self) -> float:
@@ -169,33 +125,8 @@ class HomeKitEcobeeFanModeNumber(CharacteristicEntity, NumberEntity):
 
     async def async_set_native_value(self, value: float) -> None:
         """Set the characteristic to this value."""
-
-        # Sending the fan mode request sometimes ends up getting ignored by ecobee
-        # and this might be because it the older value instead of newer, and ecobee
-        # thinks there is nothing to do.
-        # So in order to make sure that the request is executed by ecobee, we need
-        # to send a different value before sending the target value.
-        # Fan mode value is a value from 0 to 100. We send a value off by 1 first.
-
-        if value > self.min_value:
-            other_value = value - 1
-        else:
-            other_value = self.min_value + 1
-
-        if value != other_value:
-            await self.async_put_characteristics(
-                {
-                    self._char.type: other_value,
-                }
-            )
-
         await self.async_put_characteristics(
             {
                 self._char.type: value,
             }
         )
-
-
-NUMBER_ENTITY_CLASSES: dict[str, type] = {
-    CharacteristicsTypes.VENDOR_ECOBEE_FAN_WRITE_SPEED: HomeKitEcobeeFanModeNumber,
-}
