@@ -1,54 +1,71 @@
 """Support for KNX/IP fans."""
 from __future__ import annotations
 
-from collections.abc import Iterable
 import math
-from typing import Any, Callable
+from typing import Any, Final
 
+from xknx import XKNX
 from xknx.devices import Fan as XknxFan
 
-from homeassistant.components.fan import SUPPORT_OSCILLATE, SUPPORT_SET_SPEED, FanEntity
+from homeassistant import config_entries
+from homeassistant.components.fan import FanEntity, FanEntityFeature
+from homeassistant.const import CONF_ENTITY_CATEGORY, CONF_NAME, Platform
 from homeassistant.core import HomeAssistant
-from homeassistant.helpers.entity import Entity
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType
 from homeassistant.util.percentage import (
     int_states_in_range,
     percentage_to_ranged_value,
     ranged_value_to_percentage,
 )
 
-from .const import DOMAIN
+from .const import DATA_KNX_CONFIG, DOMAIN, KNX_ADDRESS
 from .knx_entity import KnxEntity
+from .schema import FanSchema
 
-DEFAULT_PERCENTAGE = 50
+DEFAULT_PERCENTAGE: Final = 50
 
 
-async def async_setup_platform(
+async def async_setup_entry(
     hass: HomeAssistant,
-    config: ConfigType,
-    async_add_entities: Callable[[Iterable[Entity]], None],
-    discovery_info: DiscoveryInfoType | None = None,
+    config_entry: config_entries.ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
 ) -> None:
-    """Set up fans for KNX platform."""
-    entities = []
-    for device in hass.data[DOMAIN].xknx.devices:
-        if isinstance(device, XknxFan):
-            entities.append(KNXFan(device))
-    async_add_entities(entities)
+    """Set up fan(s) for KNX platform."""
+    xknx: XKNX = hass.data[DOMAIN].xknx
+    config: list[ConfigType] = hass.data[DATA_KNX_CONFIG][Platform.FAN]
+
+    async_add_entities(KNXFan(xknx, entity_config) for entity_config in config)
 
 
 class KNXFan(KnxEntity, FanEntity):
     """Representation of a KNX fan."""
 
-    def __init__(self, device: XknxFan) -> None:
+    _device: XknxFan
+
+    def __init__(self, xknx: XKNX, config: ConfigType) -> None:
         """Initialize of KNX fan."""
-        self._device: XknxFan
-        super().__init__(device)
-        self._unique_id = f"{self._device.speed.group_address}"
-        self._step_range: tuple[int, int] | None = None
-        if device.max_step:
-            # FanSpeedMode.STEP:
-            self._step_range = (1, device.max_step)
+        max_step = config.get(FanSchema.CONF_MAX_STEP)
+        super().__init__(
+            device=XknxFan(
+                xknx,
+                name=config[CONF_NAME],
+                group_address_speed=config.get(KNX_ADDRESS),
+                group_address_speed_state=config.get(FanSchema.CONF_STATE_ADDRESS),
+                group_address_oscillation=config.get(
+                    FanSchema.CONF_OSCILLATION_ADDRESS
+                ),
+                group_address_oscillation_state=config.get(
+                    FanSchema.CONF_OSCILLATION_STATE_ADDRESS
+                ),
+                max_step=max_step,
+            )
+        )
+        # FanSpeedMode.STEP if max_step is set
+        self._step_range: tuple[int, int] | None = (1, max_step) if max_step else None
+        self._attr_entity_category = config.get(CONF_ENTITY_CATEGORY)
+
+        self._attr_unique_id = str(self._device.speed.group_address)
 
     async def async_set_percentage(self, percentage: int) -> None:
         """Set the speed of the fan, as a percentage."""
@@ -61,10 +78,10 @@ class KNXFan(KnxEntity, FanEntity):
     @property
     def supported_features(self) -> int:
         """Flag supported features."""
-        flags = SUPPORT_SET_SPEED
+        flags: int = FanEntityFeature.SET_SPEED
 
         if self._device.supports_oscillation:
-            flags |= SUPPORT_OSCILLATE
+            flags |= FanEntityFeature.OSCILLATE
 
         return flags
 
@@ -89,7 +106,6 @@ class KNXFan(KnxEntity, FanEntity):
 
     async def async_turn_on(
         self,
-        speed: str | None = None,
         percentage: int | None = None,
         preset_mode: str | None = None,
         **kwargs: Any,
