@@ -1,51 +1,45 @@
 """Support for LG TV running on NetCast 3 or 4."""
-from datetime import datetime, timedelta
+from __future__ import annotations
 
-from pylgnetcast import LgNetCastClient, LgNetCastError
+from datetime import datetime
+from typing import Any
+
+from pylgnetcast import LG_COMMAND, LgNetCastClient, LgNetCastError
 from requests import RequestException
 import voluptuous as vol
 
-from homeassistant import util
-from homeassistant.components.media_player import PLATFORM_SCHEMA, MediaPlayerEntity
-from homeassistant.components.media_player.const import (
-    MEDIA_TYPE_CHANNEL,
-    SUPPORT_NEXT_TRACK,
-    SUPPORT_PAUSE,
-    SUPPORT_PLAY,
-    SUPPORT_PREVIOUS_TRACK,
-    SUPPORT_SELECT_SOURCE,
-    SUPPORT_TURN_OFF,
-    SUPPORT_TURN_ON,
-    SUPPORT_VOLUME_MUTE,
-    SUPPORT_VOLUME_STEP,
-)
-from homeassistant.const import (
-    CONF_ACCESS_TOKEN,
-    CONF_HOST,
-    CONF_NAME,
-    STATE_OFF,
-    STATE_PAUSED,
-    STATE_PLAYING,
+from homeassistant.components.media_player import (
+    PLATFORM_SCHEMA,
+    MediaPlayerDeviceClass,
+    MediaPlayerEntity,
+    MediaPlayerEntityFeature,
+    MediaPlayerState,
+    MediaType,
 )
+from homeassistant.const import CONF_ACCESS_TOKEN, CONF_HOST, CONF_NAME
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.script import Script
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
+
+from .const import DOMAIN
 
 DEFAULT_NAME = "LG TV Remote"
 
 CONF_ON_ACTION = "turn_on_action"
 
-MIN_TIME_BETWEEN_FORCED_SCANS = timedelta(seconds=1)
-MIN_TIME_BETWEEN_SCANS = timedelta(seconds=10)
-
 SUPPORT_LGTV = (
-    SUPPORT_PAUSE
-    | SUPPORT_VOLUME_STEP
-    | SUPPORT_VOLUME_MUTE
-    | SUPPORT_PREVIOUS_TRACK
-    | SUPPORT_NEXT_TRACK
-    | SUPPORT_TURN_OFF
-    | SUPPORT_SELECT_SOURCE
-    | SUPPORT_PLAY
+    MediaPlayerEntityFeature.PAUSE
+    | MediaPlayerEntityFeature.VOLUME_STEP
+    | MediaPlayerEntityFeature.VOLUME_SET
+    | MediaPlayerEntityFeature.VOLUME_MUTE
+    | MediaPlayerEntityFeature.PREVIOUS_TRACK
+    | MediaPlayerEntityFeature.NEXT_TRACK
+    | MediaPlayerEntityFeature.TURN_OFF
+    | MediaPlayerEntityFeature.SELECT_SOURCE
+    | MediaPlayerEntityFeature.PLAY
+    | MediaPlayerEntityFeature.PLAY_MEDIA
 )
 
 PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
@@ -58,17 +52,21 @@ PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
 )
 
 
-def setup_platform(hass, config, add_entities, discovery_info=None):
+def setup_platform(
+    hass: HomeAssistant,
+    config: ConfigType,
+    add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up the LG TV platform."""
 
     host = config.get(CONF_HOST)
     access_token = config.get(CONF_ACCESS_TOKEN)
-    name = config.get(CONF_NAME)
+    name = config[CONF_NAME]
     on_action = config.get(CONF_ON_ACTION)
 
     client = LgNetCastClient(host, access_token)
-    domain = __name__.split(".")[-2]
-    on_action_script = Script(hass, on_action, name, domain) if on_action else None
+    on_action_script = Script(hass, on_action, name, DOMAIN) if on_action else None
 
     add_entities([LgTVDevice(client, name, on_action_script)], True)
 
@@ -76,6 +74,9 @@ def setup_platform(hass, config, add_entities, discovery_info=None):
 class LgTVDevice(MediaPlayerEntity):
     """Representation of a LG TV."""
 
+    _attr_device_class = MediaPlayerDeviceClass.TV
+    _attr_media_content_type = MediaType.CHANNEL
+
     def __init__(self, client, name, on_action_script):
         """Initialize the LG TV device."""
         self._client = client
@@ -85,6 +86,7 @@ class LgTVDevice(MediaPlayerEntity):
         # Assume that the TV is in Play mode
         self._playing = True
         self._volume = 0
+        self._channel_id = None
         self._channel_name = ""
         self._program_name = ""
         self._state = None
@@ -98,26 +100,25 @@ class LgTVDevice(MediaPlayerEntity):
             with self._client as client:
                 client.send_command(command)
         except (LgNetCastError, RequestException):
-            self._state = STATE_OFF
+            self._state = MediaPlayerState.OFF
 
-    @util.Throttle(MIN_TIME_BETWEEN_SCANS, MIN_TIME_BETWEEN_FORCED_SCANS)
-    def update(self):
+    def update(self) -> None:
         """Retrieve the latest data from the LG TV."""
 
         try:
             with self._client as client:
-                self._state = STATE_PLAYING
-                volume_info = client.query_data("volume_info")
-                if volume_info:
-                    volume_info = volume_info[0]
-                    self._volume = float(volume_info.find("level").text)
-                    self._muted = volume_info.find("mute").text == "true"
+                self._state = MediaPlayerState.PLAYING
+
+                self.__update_volume()
 
                 channel_info = client.query_data("cur_channel")
                 if channel_info:
                     channel_info = channel_info[0]
+                    channel_id = channel_info.find("major")
                     self._channel_name = channel_info.find("chname").text
                     self._program_name = channel_info.find("progName").text
+                    if channel_id is not None:
+                        self._channel_id = int(channel_id.text)
                     if self._channel_name is None:
                         self._channel_name = channel_info.find("inputSourceName").text
                     if self._program_name is None:
@@ -133,14 +134,22 @@ class LgTVDevice(MediaPlayerEntity):
                     self._sources = dict(zip(channel_names, channel_list))
                     # sort source names by the major channel number
                     source_tuples = [
-                        (k, self._sources[k].find("major").text) for k in self._sources
+                        (k, source.find("major").text)
+                        for k, source in self._sources.items()
                     ]
                     sorted_sources = sorted(
                         source_tuples, key=lambda channel: int(channel[1])
                     )
                     self._source_names = [n for n, k in sorted_sources]
         except (LgNetCastError, RequestException):
-            self._state = STATE_OFF
+            self._state = MediaPlayerState.OFF
+
+    def __update_volume(self):
+        volume_info = self._client.get_volume()
+        if volume_info:
+            (volume, muted) = volume_info
+            self._volume = volume
+            self._muted = muted
 
     @property
     def name(self):
@@ -173,9 +182,9 @@ class LgTVDevice(MediaPlayerEntity):
         return self._source_names
 
     @property
-    def media_content_type(self):
-        """Content type of current playing media."""
-        return MEDIA_TYPE_CHANNEL
+    def media_content_id(self):
+        """Content id of current playing media."""
+        return self._channel_id
 
     @property
     def media_channel(self):
@@ -191,7 +200,7 @@ class LgTVDevice(MediaPlayerEntity):
     def supported_features(self):
         """Flag media player features that are supported."""
         if self._on_action_script:
-            return SUPPORT_LGTV | SUPPORT_TURN_ON
+            return SUPPORT_LGTV | MediaPlayerEntityFeature.TURN_ON
         return SUPPORT_LGTV
 
     @property
@@ -201,54 +210,71 @@ class LgTVDevice(MediaPlayerEntity):
             f"{self._client.url}data?target=screen_image&_={datetime.now().timestamp()}"
         )
 
-    def turn_off(self):
+    def turn_off(self) -> None:
         """Turn off media player."""
-        self.send_command(1)
+        self.send_command(LG_COMMAND.POWER)
 
-    def turn_on(self):
+    def turn_on(self) -> None:
         """Turn on the media player."""
         if self._on_action_script:
             self._on_action_script.run(context=self._context)
 
-    def volume_up(self):
+    def volume_up(self) -> None:
         """Volume up the media player."""
-        self.send_command(24)
+        self.send_command(LG_COMMAND.VOLUME_UP)
 
-    def volume_down(self):
+    def volume_down(self) -> None:
         """Volume down media player."""
-        self.send_command(25)
+        self.send_command(LG_COMMAND.VOLUME_DOWN)
 
-    def mute_volume(self, mute):
+    def set_volume_level(self, volume: float) -> None:
+        """Set volume level, range 0..1."""
+        self._client.set_volume(float(volume * 100))
+
+    def mute_volume(self, mute: bool) -> None:
         """Send mute command."""
-        self.send_command(26)
+        self.send_command(LG_COMMAND.MUTE_TOGGLE)
 
-    def select_source(self, source):
+    def select_source(self, source: str) -> None:
         """Select input source."""
         self._client.change_channel(self._sources[source])
 
-    def media_play_pause(self):
+    def media_play_pause(self) -> None:
         """Simulate play pause media player."""
         if self._playing:
             self.media_pause()
         else:
             self.media_play()
 
-    def media_play(self):
+    def media_play(self) -> None:
         """Send play command."""
         self._playing = True
-        self._state = STATE_PLAYING
-        self.send_command(33)
+        self._state = MediaPlayerState.PLAYING
+        self.send_command(LG_COMMAND.PLAY)
 
-    def media_pause(self):
+    def media_pause(self) -> None:
         """Send media pause command to media player."""
         self._playing = False
-        self._state = STATE_PAUSED
-        self.send_command(34)
+        self._state = MediaPlayerState.PAUSED
+        self.send_command(LG_COMMAND.PAUSE)
 
-    def media_next_track(self):
+    def media_next_track(self) -> None:
         """Send next track command."""
-        self.send_command(36)
+        self.send_command(LG_COMMAND.FAST_FORWARD)
 
-    def media_previous_track(self):
+    def media_previous_track(self) -> None:
         """Send the previous track command."""
-        self.send_command(37)
+        self.send_command(LG_COMMAND.REWIND)
+
+    def play_media(self, media_type: str, media_id: str, **kwargs: Any) -> None:
+        """Tune to channel."""
+        if media_type != MediaType.CHANNEL:
+            raise ValueError(f"Invalid media type: {media_type}")
+
+        for name, channel in self._sources.items():
+            channel_id = channel.find("major")
+            if channel_id is not None and int(channel_id.text) == int(media_id):
+                self.select_source(name)
+                return
+
+        raise ValueError(f"Invalid media id: {media_id}")
