@@ -5,15 +5,17 @@ import voluptuous as vol
 from youtube_dl import YoutubeDL
 from youtube_dl.utils import DownloadError, ExtractorError
 
-from homeassistant.components.media_player import MEDIA_PLAYER_PLAY_MEDIA_SCHEMA
-from homeassistant.components.media_player.const import (
+from homeassistant.components.media_player import (
     ATTR_MEDIA_CONTENT_ID,
     ATTR_MEDIA_CONTENT_TYPE,
     DOMAIN as MEDIA_PLAYER_DOMAIN,
+    MEDIA_PLAYER_PLAY_MEDIA_SCHEMA,
     SERVICE_PLAY_MEDIA,
 )
 from homeassistant.const import ATTR_ENTITY_ID
+from homeassistant.core import HomeAssistant, ServiceCall
 from homeassistant.helpers import config_validation as cv
+from homeassistant.helpers.typing import ConfigType
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -38,10 +40,10 @@ CONFIG_SCHEMA = vol.Schema(
 )
 
 
-def setup(hass, config):
+def setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up the media extractor service."""
 
-    def play_media(call):
+    def play_media(call: ServiceCall) -> None:
         """Get stream URL and send it to the play_media service."""
         MediaExtractor(hass, config[DOMAIN], call.data).extract_and_send()
 
@@ -89,9 +91,7 @@ class MediaExtractor:
                 "Could not retrieve data for the URL: %s", self.get_media_url()
             )
         else:
-            entities = self.get_entities()
-
-            if not entities:
+            if not (entities := self.get_entities()):
                 self.call_media_player_service(stream_selector, None)
 
             for entity_id in entities:
