@@ -7,13 +7,16 @@ ABBREVIATIONS = {
     "aux_cmd_t": "aux_command_topic",
     "aux_stat_tpl": "aux_state_template",
     "aux_stat_t": "aux_state_topic",
+    "av_tones": "available_tones",
     "avty": "availability",
     "avty_mode": "availability_mode",
     "avty_t": "availability_topic",
+    "avty_tpl": "availability_template",
     "away_mode_cmd_t": "away_mode_command_topic",
     "away_mode_stat_tpl": "away_mode_state_template",
     "away_mode_stat_t": "away_mode_state_topic",
     "b_tpl": "blue_template",
+    "bri_cmd_tpl": "brightness_command_template",
     "bri_cmd_t": "brightness_command_topic",
     "bri_scl": "brightness_scale",
     "bri_stat_t": "brightness_state_topic",
@@ -39,13 +42,16 @@ ABBREVIATIONS = {
     "cmd_tpl": "command_template",
     "cod_arm_req": "code_arm_required",
     "cod_dis_req": "code_disarm_required",
+    "cod_trig_req": "code_trigger_required",
     "curr_temp_t": "current_temperature_topic",
     "curr_temp_tpl": "current_temperature_template",
     "dev": "device",
     "dev_cla": "device_class",
     "dock_t": "docked_topic",
     "dock_tpl": "docked_template",
+    "e": "encoding",
     "en": "enabled_by_default",
+    "ent_cat": "entity_category",
     "err_t": "error_topic",
     "err_tpl": "error_template",
     "fanspd_t": "fan_speed_topic",
@@ -53,6 +59,7 @@ ABBREVIATIONS = {
     "fanspd_lst": "fan_speed_list",
     "flsh_tlng": "flash_time_long",
     "flsh_tsht": "flash_time_short",
+    "fx_cmd_tpl": "effect_command_template",
     "fx_cmd_t": "effect_command_topic",
     "fx_list": "effect_list",
     "fx_stat_t": "effect_state_topic",
@@ -73,6 +80,7 @@ ABBREVIATIONS = {
     "hs_stat_t": "hs_state_topic",
     "hs_val_tpl": "hs_value_template",
     "ic": "icon",
+    "img_e": "image_encoding",
     "init": "initial",
     "hum_cmd_t": "target_humidity_command_topic",
     "hum_cmd_tpl": "target_humidity_command_template",
@@ -91,12 +99,14 @@ ABBREVIATIONS = {
     "min_mirs": "min_mireds",
     "max_temp": "max_temp",
     "min_temp": "min_temp",
+    "mode": "mode",
     "mode_cmd_tpl": "mode_command_template",
     "mode_cmd_t": "mode_command_topic",
     "mode_stat_t": "mode_state_topic",
     "mode_stat_tpl": "mode_state_template",
     "modes": "modes",
     "name": "name",
+    "obj_id": "object_id",
     "off_dly": "off_delay",
     "on_cmd_type": "on_command_type",
     "ops": "options",
@@ -113,6 +123,7 @@ ABBREVIATIONS = {
     "pl_arm_away": "payload_arm_away",
     "pl_arm_home": "payload_arm_home",
     "pl_arm_nite": "payload_arm_night",
+    "pl_arm_vacation": "payload_arm_vacation",
     "pl_arm_custom_b": "payload_arm_custom_bypass",
     "pl_avail": "payload_available",
     "pl_cln_sp": "payload_clean_spot",
@@ -133,6 +144,8 @@ ABBREVIATIONS = {
     "pl_osc_off": "payload_oscillation_off",
     "pl_osc_on": "payload_oscillation_on",
     "pl_paus": "payload_pause",
+    "pl_prs": "payload_press",
+    "pl_rst": "payload_reset",
     "pl_rst_hum": "payload_reset_humidity",
     "pl_rst_mode": "payload_reset_mode",
     "pl_rst_pct": "payload_reset_percentage",
@@ -143,6 +156,7 @@ ABBREVIATIONS = {
     "pl_ret": "payload_return_to_base",
     "pl_toff": "payload_turn_off",
     "pl_ton": "payload_turn_on",
+    "pl_trig": "payload_trigger",
     "pl_unlk": "payload_unlock",
     "pos_clsd": "position_closed",
     "pos_open": "position_open",
@@ -197,6 +211,8 @@ ABBREVIATIONS = {
     "stat_val_tpl": "state_value_template",
     "step": "step",
     "stype": "subtype",
+    "sup_dur": "support_duration",
+    "sup_vol": "support_volume_set",
     "sup_feat": "supported_features",
     "sup_clrm": "supported_color_modes",
     "swing_mode_cmd_tpl": "swing_mode_command_template",
@@ -243,10 +259,12 @@ ABBREVIATIONS = {
 
 DEVICE_ABBREVIATIONS = {
     "cns": "connections",
+    "cu": "configuration_url",
     "ids": "identifiers",
     "name": "name",
     "mf": "manufacturer",
     "mdl": "model",
+    "hw": "hw_version",
     "sw": "sw_version",
     "sa": "suggested_area",
 }
